/*
 * procedures for pointing_proc
 * Time-stamp: <2005-02-12 19:08:44 dimm>
 */
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <gtk/gtk.h>

#include "seemon2.h"
#include "protocol.h"

/*
 * mseemon.is_pointing
 * 0: not in pointing mode
 * 1: select new star
 * 2: point meade
 * 3: wait for star image in finder fixed
 * 4: move star to target position in finder image
 * 5: check star is in I.I. image
 * 6: centering in I.I. image
 */

/* procedure 1: select a new star */
int pointproc01(void) {
  char str[20];
  int ret;

  switch (mseemon.op_ssmode) {
  case 0: /* autoselect */
    ret = autoselect_star();
    if (ret == EXIT_SUCCESS) {
      /* if succeeded, set is_pointing to 2  */
      /* v0.6.9 20050112 added check for long sleeping */
      if (mseemon.is_sleeping == 0) {
	mseemon.is_pointing = 2;
#ifdef DEBUG
      printf("### is_pointing ==> 2 autopoint_procedure.c 38\n");
#endif
      }
    }
    break;
  case 1: /* point to currently selected */
    /* check star is selected  */
    if (mseemon.selnum < 0) {
      sprintf(str, "star not selected!\n");
      mydialog(str);
      mseemon.is_pointing = 0;
      return(EXIT_FAILURE);
    }
    printf("# pointing_proc01: selected star: %d ntry1=%d\n", mseemon.selnum, mseemon.ntry1);
    if (fout != NULL) {
      fprintf(fout, "# pointing_proc01: selected star: %d ntry1=%d.\n", mseemon.selnum, mseemon.ntry1);
    }
    ret = EXIT_SUCCESS;
    mseemon.is_pointing = 2;
#ifdef DEBUG
      printf("### is_pointing ==> 2 autopoint_procedure.c 57\n");
#endif
    break;
  case 2: /* already pointed */
    mseemon.is_pointing = 3;
    mseemon.tel_stat = 3; /* pointed  */
    break;
  default:
    printf("# invalid star selection mode.\n");
    mseemon.is_pointing = 0;
    return(EXIT_FAILURE);
  }

  return(EXIT_SUCCESS);
}

/* procedure2: point meade */
int pointproc02(void) {
  int ret;

  /* send command to meade */
  if ((ret = point_meade()) == EXIT_SUCCESS) {
    /* succeeded */
    mseemon.is_pointing ++;
    mseemon.tel_stat = 3; /* pointed  */
  } else {
    printf("# pointing_proc02 failed in pointing meade.\n");
      if (fout != NULL) {
	fprintf(fout, "# pointing_proc02 failed in pointing meade.\n");
      }
      return(EXIT_FAILURE); /* failed */
  }
  return(EXIT_SUCCESS);
}

/* procedure3: wait star position in finder fixed */
int pointproc03(void) {
  int ret;

  ret = wait_finder_fixed();
  if (ret == EXIT_SUCCESS) {
    /* succeeded */
    #ifdef DEBUG
    printf("# pointing_proc03(): wait_finder_fixed succeeded.\n");
    #endif
    mseemon.is_pointing++;
    mseemon.proc_cnt = 0; /* init counter */
  } else {
    printf("# pointing_proc03() failed in fixing finder position.\n");
      if (fout != NULL) {
	fprintf(fout, "# pointing_proc03() failed in fixing finder position.\n");
      }
      return(EXIT_FAILURE); /* failed */
  }
  return(EXIT_SUCCESS);
}

/* procedure4: move star to target position in finder image */
int pointproc04(void) {
  int ret;

  /* move star to target position in finder image */
  ret = point_finder_target(); /* in telctl_automatic.c */

  if (ret == -1) {
    /* succeeded */
    mseemon.is_pointing++;
    mseemon.proc_cnt = 0; /* reset counter */
    mseemon.finder_dist_prev = 1000;
  }
  if (ret == EXIT_FAILURE) {
    printf("# pointing_proc04() failed in setting star to the target position.\n");
    if (fout != NULL) {
      fprintf(fout, "# pointing_proc04() failed in setting star to the target position.\n");
    }
    return(EXIT_FAILURE); /* failed */
  }
  /* otherwise (ret == 0), more motion would be needed
   * sleep for a while and try again
   */
  if (ret == 0) {
    #ifdef DEBUG
    printf("# pointproc04(): dist:%d prev:%d\n", mseemon.finder_dist, mseemon.finder_dist_prev);
    #endif
    if (mseemon.finder_dist > mseemon.finder_dist_prev - 5) {
      /* no significant approach to target position.
       * increment counter. v0.7.2 20050211
       */
      mseemon.proc_cnt++;
      if (mseemon.proc_cnt > MAX_TRY_POINTING) {
	printf("# pointing_proc04 failed: timeout while trying move to target position in finder.\n");
	if (fout != NULL) {
	  fprintf(fout, "# pointing_proc04 failed: timeout while trying move to target position in finder.\n");
	}
	return(EXIT_FAILURE); /* failed */
      }
    }
    mseemon.finder_dist_prev = mseemon.finder_dist;
    printf("# pointing_proc04: trying to move star to target position in finder. (%d time(s))\n", mseemon.proc_cnt);
    sleep(2);
  }

  return(EXIT_SUCCESS);
}

/* procedure5: check star is in I.I. image */
int pointproc05(void) {
  int i, ret, flg = 0;

  dvgrab();
  load_pix_buffer();
  redraw_image();

  copy_to_gray_buffer();
  if ((ret = get_stars()) == EXIT_SUCCESS) { /* in find_star.c */
    for (i=0; i<2; i++) {
      search_peak(i); /* in get_cg.c */
      calc_cg(i);     /* in get_cg.c, in order to get peakval */
    }
    if (mseemon.peakval[0] > MIN_PEAK &&
	mseemon.peakval[1] > MIN_PEAK) {
      mseemon.is_pointing++; /* succeeded */
      mseemon.proc_cnt = 0;  /* reset counter */
      flg = 1;
    }
  }

  if (!flg) {
    mseemon.proc_cnt++;
    if (mseemon.proc_cnt >= MAX_TRY_POINTING) {
      printf("# pointing_proc05 failed while checking stars in I.I. image.\n");
      if (fout != NULL) {
	fprintf(fout, "# pointing_proc05 failed while checking stars in I.I. image.\n");
      }
      return(EXIT_FAILURE); /* failed */
    }
  }

  return(EXIT_SUCCESS);
}

/* procedure6: centering in I.I. image */
int pointproc06(void) {
  int i, ret;

  printf("# pointing_proc06 called.\n");
  if (fout != NULL) {
    fprintf(fout, "# pointing_proc06() called.\n");
  }
  if ((ret = is_cloudy()) == 0) {
    mseemon.is_guiding = 1;
    start_guid(); /* in guid.c */

    if (mseemon.is_guiding == 0) {
      mseemon.is_pointing = 0;
      printf("# pointing_proc() successfully finished.\n");
      if (fout != NULL) {
	fprintf(fout, "# pointing_proc() successfully finished.\n");
      }
      /* set IICCD shutter time to default value */
      mseemon.time_iishutter = mseemon.time_iishutter0;
      set_iishutter(mseemon.time_iishutter);
      /* reset peak_average */
      reset_peak_history(); /* in cap_window.c */

      /* reset star flags */
      reset_obj_flags(); /* in obj.c */
      mseemon.ntry1 = 0;
      mseemon.n_average = 0;
    } else {
      dvgrab(); /* capture from dv */

      if ((load_pix_buffer())) { /* load to pixbuf */
	printf("# error in reading from capture image.\n");
	if (fout != NULL) {
	  fprintf(fout, "# pointproc06: error in reading from capture image.\n");
	}
	return(EXIT_FAILURE);
      }

      if (!(copy_to_gray_buffer())) {
	/* find star position */
	get_stars();

	/* calc center of gravity */
	/* check rects are set */
	for (i=0; i<2; i++) {
	  search_peak(i);
	  calc_cg(i);
	}

	/* check stars are found */
	if (mseemon.peakval[0] > mseemon.min_peak &&
	    mseemon.peakval[1] > mseemon.min_peak) {
	  /* stars seem to be in */
	  /* follow cg */
	  mseemon.star_center[0].cx = (int)(mseemon.cg[0].x+0.5);
	  mseemon.star_center[0].cy = (int)(mseemon.cg[0].y+0.5);
	  mseemon.star_center[1].cx = (int)(mseemon.cg[1].x+0.5);
	  mseemon.star_center[1].cy = (int)(mseemon.cg[1].y+0.5);

	  printf("# pointproc06: stars seems to be at edge of  I.I.\n");
	  if (fout != NULL) {
	    fprintf(fout, "# pointproc06: stars seems to be at edge of  I.I.\n");
	  }

	  redraw_image();
	} else {
	  printf("# pointingproc06: seems to lost star(s)..\n");
	  printf("# call recover_stars().\n");
	  if (fout != NULL) {
	    fprintf(fout, "# pointproc06: seems to lost star(s)..\n");
	    fprintf(fout, "# call recover_stars().\n");
	  }
	  redraw_image();
	  ret = recover_stars(); /* in telctl_automatic.c */

	  if (ret == EXIT_FAILURE) {
	    printf("# pointproc06: recover_stars failed.\n");
	    printf("# pointing again.\n");
	    if (fout != NULL) {
	      fprintf(fout, "# pointproc06: recover_stars failed.\n");
	      fprintf(fout, "# pointing again.\n");
	    }
	    mseemon.is_pointing = 2;
#ifdef DEBUG
      printf("### is_pointing ==> 2 autopoint_procedure.c 268\n");
#endif
	  } else {
	    printf("# pointingproc06: recover_stars succeeded.\n");
	    printf("# setting is_pointing=5.\n");
	    if (fout != NULL) {
	      fprintf(fout, "# pointproc06: recover_stars succeeded.\n");
	      fprintf(fout, "# setting is_pointing=5.\n");
	    }
	    mseemon.is_pointing = 5;
	  }
	}
      }
    }
  } else {
    printf("# pointproc06: seems cloudy.\n");
    if (fout != NULL) {
      fprintf(fout, "# pointproc06: seems cloudy.\n");
    }
  }

  return(EXIT_SUCCESS);
}
