/* 
 * image capture window
 * and capturing from DV
 *
 * 2002.10.26 -
 * Time-stamp: <2005-12-13 15:20:37 dimm>
 *
 *
 * captured image is stored in gunchar* graybuf
 *  as a 8-bit gray-scale image.
 *  width and height are in int wid, hgt
 *
 */
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "seemon2.h"

GdkPixbuf *pixbuf;

/* functions in this file */
void dvgrab(void);                 /* image read from DV */
int dvgrab_seq(void);             /* read image sequence from DV */
int load_pix_buffer(void);         /* load pix buffer */
int load_pix_buffer2(char* fpath); /* load pix buffer */
int copy_to_gray_buffer(void);     /* copy from pixbuf to gray buffer */
int init_cap_window(GtkWidget *window_cap); /* initialization of capture window */
int draw_rect(GtkWidget *w, int n);         /* draw rectangle */
int draw_cg(GtkWidget *w, int n);           /* draw center of gravity */
void redraw_image (void);          /* redraw image */
void display_r0 (void);            /* output resultant r0 and FWHM */
void set_lamp(GtkWidget *widget, int color); /* v0.4.0u */


/* image read from DV
 *
 * NOTE: this is preliminary [2002.11.29]
 */
void dvgrab(void){
  /* dvgrab; update v0.4.0 2003.05.16 */
  /* renamed v0.4.3 2003.05.29 */
  //  system("/usr/local/bin/grab_single3.sh");

  /* v0.5.7 */
  system("/usr/local/bin/grab");
}

int dvgrab_seq(void) {
  int ret;
  //  system("/usr/local/bin/grabdv.sh");
  /* v0.5.7 */
  ret = system("/usr/local/bin/grabseq");
  return(ret);
}


/* load pix buffer */
int load_pix_buffer(void) {

  /* release and destroy previous pixbuf */
  if (pixbuf != NULL) {
    g_object_unref(pixbuf);
  }

  /* load from file
   *  preliminary.
   *  read from .ppm file made by "transcode" command.
   */  
  if ((pixbuf = gdk_pixbuf_new_from_file(mseemon.dvfilepath, NULL)) == NULL) {
    printf("failed opening ppm file. path:%s\n", mseemon.dvfilepath);
    return(EXIT_FAILURE);
  }

  /* commented out in 2003.02.11. this seems to cause
   * memory leak */ 
  /*  g_object_ref(pixbuf); /* add reference */
  wid = gdk_pixbuf_get_width(pixbuf);
  hgt = gdk_pixbuf_get_height(pixbuf);

  return(EXIT_SUCCESS);
}

/* load pix buffer2
 * get file path from argument
 * 2003.05.17
 */
int load_pix_buffer2(char* fpath) {

  /* release and destroy previous pixbuf */
  if (pixbuf != NULL) {
    g_object_unref(pixbuf);
  }

  /* load from file
   *  preliminary.
   *  read from .ppm file made by "transcode" command.
   */  
  if ((pixbuf = gdk_pixbuf_new_from_file(fpath, NULL)) == NULL) {
    printf("failed opening ppm file. path:%s\n", fpath);
    return(EXIT_FAILURE);
  }

  wid = gdk_pixbuf_get_width(pixbuf);
  hgt = gdk_pixbuf_get_height(pixbuf);

  return(EXIT_SUCCESS);
}


/* copy from pixbuf 
 * to gray buffer
 */
int copy_to_gray_buffer(void) {
  int i, j;
  guchar *inpix, *outpix;

  if ((inpix = gdk_pixbuf_get_pixels(pixbuf)) == NULL) {
    printf("error in reading pixbuf.\n");
    return(EXIT_FAILURE);
  }

  outpix = graybuf;

  for (i = 0; i < hgt; i++) {
    for (j = 0; j < wid; j++) {
      //      *outpix = *inpix;
      /* modified v0.5.8 2004.02.23 */
      *outpix = (guchar)((*inpix + *(inpix+1) + *(inpix+2))/3);
      inpix += 3;
      outpix ++;
    }
  }

  return(EXIT_SUCCESS);
}

/* draw rectangle */
int draw_rect(GtkWidget *w, int n) {
  int tx1, tx2, ty1, ty2;

  tx1 = mseemon.star_center[n].cx - mseemon.star_rect[n].cx/2;
  if (tx1 < 0) tx1 = 0;
  tx2 = mseemon.star_center[n].cx + mseemon.star_rect[n].cx/2;
  if (tx2 > wid) tx2 = wid;
  ty1 = mseemon.star_center[n].cy - mseemon.star_rect[n].cy/2;
  if (ty1 < 0) ty1 = 0;
  ty2 = mseemon.star_center[n].cy + mseemon.star_rect[n].cy/2;
  if (ty2 > hgt) ty2 = hgt;

  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx1, ty1, tx1, ty2);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx2, ty1, tx2, ty2);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx1, ty1, tx2, ty1);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx1, ty2, tx2, ty2);

}

/* draw center of gravity points */
int draw_cg(GtkWidget *w, int n) {
  int tx1, tx2, ty1, ty2;

  tx1 = (int)(mseemon.cg[n].x +0.5) -1;
  tx2 = (int)(mseemon.cg[n].x +0.5) +1;
  ty1 = (int)(mseemon.cg[n].y +0.5) -1;
  ty2 = (int)(mseemon.cg[n].y +0.5) +1;

  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->black_gc, tx1, ty1, tx1, ty2);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->black_gc, tx2, ty1, tx2, ty2);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->black_gc, tx1, ty1, tx2, ty1);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->black_gc, tx1, ty2, tx2, ty2);

  /* cg_rect added ver.0.2.6 */
  tx1 = mseemon.peak[n].cx - mseemon.cg_rect[n].cx/2;
  if (tx1 < 0) tx1 = 0;
  tx2 = mseemon.peak[n].cx + mseemon.cg_rect[n].cx/2;
  if (tx2 > wid) tx2 = wid;
  ty1 = mseemon.peak[n].cy - mseemon.cg_rect[n].cy/2;
  if (ty1 < 0) ty1 = 0;
  ty2 = mseemon.peak[n].cy + mseemon.cg_rect[n].cy/2;
  if (ty2 > hgt) ty2 = hgt;

  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx1, ty1, tx1, ty2);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx2, ty1, tx2, ty2);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx1, ty1, tx2, ty1);
  gdk_draw_line(GDK_DRAWABLE(w->window), w->style->white_gc, tx1, ty2, tx2, ty2);


}
 
/* redraw image */
void redraw_image (void) {
  int i;

  gdk_draw_gray_image (GDK_DRAWABLE(g_image_area1->window), g_image_area1->style->fg_gc[GTK_STATE_NORMAL],
		       0, 0, wid, hgt,
		       GDK_RGB_DITHER_MAX, graybuf, wid);
  for (i=0; i<2; i++) {
    if (flg_set_rect[i]) {
      draw_rect(g_image_area1, i);
      draw_cg(g_image_area1, i); /* added v.0.4.3 */
    }
  }
}


/* reload image and update immediately 
 * added v.0.4.0u 2003.05.16
 */
void reload_image_quick(gpointer* pointer) {
  int i, ret;
  GtkWidget* w;

  w = *pointer; /* pointer for g_image_area */

  dvgrab(); /* capture from dv */

  if ((load_pix_buffer())) { /* load to pixbuf */
    printf("error in reading from capture image.\n");
    return;
  }

  if (!(copy_to_gray_buffer())) {
    /* calc center of gravity */
    /* check rects are set */
    for (i=0; i<2; i++) {
      if (flg_set_rect[i] ==1) {
	search_peak(i);
	calc_cg(i);
      }
    }

    if (flg_set_rect[0] * flg_set_rect[1]) {
      calc_r0_new();
    }

    /* redraw image
     * redrawing every time when not tracking, 
     * redraw once in time_redraw when tracking
     * v.0.3.0, 2003.05.16
     */
    if (mseemon.is_tracking) {
      /* check image distance */
      if ((ret = check_cg()) >= 1) {
	mseemon.motionflg++;
	if (mseemon.motionflg >= 3) {
	  /* distance error repeats: try get_stars */
	  get_stars();
	  for (i=0; i<2; i++) {
	    search_peak(i);
	    calc_cg(i);
	  }
	}
      } else {
	/* check_cg passed; motionflg set to 0 */
	mseemon.motionflg = 0;
      }

      if (mseemon.peakval[0] > mseemon.min_peak &&
	  mseemon.peakval[1] > mseemon.min_peak) {
	/* follow cg */
	mseemon.star_center[0].cx = (int)(mseemon.cg[0].x+0.5);
	mseemon.star_center[0].cy = (int)(mseemon.cg[0].y+0.5);
	mseemon.star_center[1].cx = (int)(mseemon.cg[1].x+0.5);
	mseemon.star_center[1].cy = (int)(mseemon.cg[1].y+0.5);
      } else {
	printf("peak count below min.; not tracking.\n");
	if (fout != NULL) {
	  fprintf(fout, "# peak count below min.; not tracking.\n");
	}
      }
    }

    redraw_image();

    display_r0(); /* display result */

    /* if automatic mode, go to analysis mode */
    /* v0.5.3 2003.10.03 */
    if (mseemon.opmode == 1) {
      ret = is_cloudy();
      if (ret == 0) {
	printf("# quick --> analysis.\n");
	if (fout != NULL) {
	  fprintf(fout, "# quick --> analysis.\n");
	}
	mseemon.is_capturing = 2;
	mseemon.is_recording = 1;
	mseemon.is_tracking = 1;
	mseemon.is_guiding = 0;
	mseemon.guideflg = 1;
	gtk_button_set_label(GTK_BUTTON(btn1), " Stop Tracking");
	gtk_button_set_label(GTK_BUTTON(btn2), " Stop Analysis");
	gtk_button_set_label(GTK_BUTTON(btn3), " Stop Recording");
	set_lamp(g_lamp_area1, 1);
	set_lamp(g_lamp_area2, 1);
	set_lamp(g_lamp_area3, 1);
      }
    }
  }

}


/* analysis mode
 * sequential DV reading and calc r0
 * 2003.05.17
 */
void reload_image(gpointer* pointer) {
  int i, j, ret;
  int flg = 0;
  GtkWidget* w;

  w = *pointer; /* pointer for g_image_area */

  ret = dvgrab_seq();
  if (ret) {
    #ifdef DEBUG
    printf("# dvgrab_seq failed: not analyizing.\n");
    #endif
    if (fout != NULL) {
      fprintf(fout, "# dvgrab_seq failed: not analyizing.\n");
    }
    return;
  }

  n_calc_r0 = 0; /* reset frame counter */

  for (j = 0; j < NSEQ; j++) {
    if ((load_pix_buffer2(mseemon.dvseqpath[j]))) { /* load to pixbuf */
      printf("error in reading from capture image.\n");
      return;
    }

    if (!(copy_to_gray_buffer())) {
      /* calc center of gravity */
      /* check rects are set */
      for (i=0; i<2; i++) {
	if (flg_set_rect[i] ==1) {
	  search_peak(i);
	  calc_cg(i);
	}
      }

      mseemon.diffcg[n_calc_r0].x = (mseemon.cg[0].x - mseemon.cg[1].x)*scalex;
      mseemon.diffcg[n_calc_r0].y = (mseemon.cg[0].y - mseemon.cg[1].y)*scaley;

      /* increase n_calc_r0 */
      n_calc_r0++;

      if (mseemon.is_tracking) {
	/* follow cg */
	mseemon.star_center[0].cx = (int)(mseemon.cg[0].x+0.5);
	mseemon.star_center[0].cy = (int)(mseemon.cg[0].y+0.5);
	mseemon.star_center[1].cx = (int)(mseemon.cg[1].x+0.5);
	mseemon.star_center[1].cy = (int)(mseemon.cg[1].y+0.5);
      }
    }
  } /* end of sequence reading loop */

  if (flg_set_rect[0] * flg_set_rect[1]) {
    if (!(cmp_diffcg(n_calc_r0))) {
      /* calc r0 */
      ana_r0();
    } else {
      #ifdef DEBUG
      printf("# diffcg did not change. skipped.\n");
      #endif
      if (fout != NULL) {
	fprintf(fout, "# diffcg did not change. skipped.\n");
      }
    }
    copy_diffcg(n_calc_r0);
  }

  if (mseemon.is_tracking) {
    /* check stars are successfully tracked */

    /* check image distance */
    if ((ret = check_cg()) >= 1) {
      #ifdef DEBUG
      printf("# image distance large: %6.2f.\n", mseemon.image_dist);
      if (fout != NULL) {
	fprintf(fout, "# image distance large: %6.2f.\n", mseemon.image_dist);
      }
      #endif
      flg = 1;
    }

    /* check two stars are identified */
    /* added v.0.4.3 2003.05.29 */
    if (mseemon.diffcg[n_calc_r0-1].x < scalex &&
	mseemon.diffcg[n_calc_r0-1].x > -1*scalex &&
	mseemon.diffcg[n_calc_r0-1].y < scaley &&
	mseemon.diffcg[n_calc_r0-1].y > -1*scaley) {
      /* one star lost */
      #ifdef DEBUG
      printf("# a stellar image seems to be lost.\n");
      if (fout != NULL) {
	fprintf(fout, "# a stellar image seems to be lost.\n");
      }
      #endif
      flg = 1;
    }

    /* check fwhm 
     * added v.0.4.8 2003.08.17 
     */
    if (mseemon.fwhm.x > FWHM_MAX &&
	mseemon.fwhm.y > FWHM_MAX) {
      printf("# FWHM too large: lost stars?\n");
      #ifdef DEBUG
      printf("# fwhmx:%6.2f fwhmy:%6.2f FWHM_MAX:%6.2f \n", 
	     mseemon.fwhm.x, mseemon.fwhm.y, FWHM_MAX);
      #endif
      if (fout != NULL) {
	fprintf(fout, "# FWHM too large: lost stars?\n");
      }
      flg = 1;
    }

    /* stars are lost. */
    if (flg) {
      /* cloudy? */
      if ((ret = is_cloudy())==-1) {
	printf("reload_image: it seems cloudy.\n");
	if (fout != NULL) {
	  fprintf(fout, "# reload_image: it seems cloudy.\n");
	}
	/* turn to quick preview mode */
	if (mseemon.opmode == 1) {
	  printf("reload_image: turns to preview mode.\n");
	  if (fout != NULL) {
	    fprintf(fout, "# reload_image: turns to preview mode.\n");
	  }
	  reset_starfinding(); /* in cap_window_callbacks.c */
	}

      } else {
	/* try get_stars() */
	get_stars(); /* in find_star.c */
	for (i=0; i<2; i++) {
	  search_peak(i);
	  calc_cg(i);
	}
	/* check stars are found */
	if (mseemon.peakval[0] > mseemon.min_peak &&
	    mseemon.peakval[1] > mseemon.min_peak) {
	  /* stars seem to be in */
	  /* follow cg */
	  mseemon.star_center[0].cx = (int)(mseemon.cg[0].x+0.5);
	  mseemon.star_center[0].cy = (int)(mseemon.cg[0].y+0.5);
	  mseemon.star_center[1].cx = (int)(mseemon.cg[1].x+0.5);
	  mseemon.star_center[1].cy = (int)(mseemon.cg[1].y+0.5);
	} else {
	  /* stars are not in I.I.
	   * call recover_stars() */
	  #ifdef DEBUG
	  printf("# star peaks: %d %d limit:%d\n", mseemon.peakval[0], mseemon.peakval[1], mseemon.min_peak);
	  #endif
	  printf("# stars are not in I.I. calling recover_stars().\n");
	  if (fout != NULL) {
	    fprintf(fout, "# stars are not in I.I. calling recover_stars().\n");
	  }
	  ret = recover_stars();
	  if (ret == EXIT_FAILURE) {
	    /* recover_stars failed */
	    /* reset (in cap_window_callbacks.c) */
	    printf("# recover_stars() failed. reset.\n");
	    if (fout != NULL) {
	      fprintf(fout, "# recover_stars() failed. reset.\n");
	    }
	    reset_starfinding();
	    /* if automatic mode, do pointing again */
	    /* v0.6.9 20050112 added check for long sleeping */
	    if (mseemon.opmode == 1 && mseemon.is_sleeping == 0) {
	      #ifdef DEBUG
	      printf("### is_pointing ==> 2 cap_window.c 487\n");
	      #endif
	      mseemon.is_pointing = 2;
	    }
	  }
	}
      }
    }
  }

  redraw_image();

  display_r0(); /* display result */
}

/* reload image for guiding mode
 * added v.0.4.1 2003.05.28
 * modified v.0.5.1 2003.10.01
 *
 * assume is_tracking == 1, 
 *        flg_set_rect[] == 1
 */
void reload_image_guiding(gpointer* pointer) {
  int i, ret, flg, cnt, cnt2, success;
  GtkWidget* w;
  gchar str[100];

  w = *pointer; /* pointer for g_image_area */

  /* guiding */
  start_guid();

  if (mseemon.ntry2 > MAX_TRY_GUIDE) {
    /* too many trials. possibly high-altitude? */
    printf("# guiding: too many trials. possibly high-altitude?\n");
    if (fout != NULL) {
      fprintf(fout, "# guiding: too many trials. possibly high-altitude?\n");
    }
    /* reset (in cap_window_callbacks.c) */
    reset_starfinding();

    /* if automatic mode do pointing again */
    if (mseemon.opmode == 1) {
	mseemon.is_pointing = 1;
    }
  } else {

    dvgrab(); /* capture from dv */

    if ((load_pix_buffer())) { /* load to pixbuf */
      printf("error in reading from capture image.\n");
      return;
    }

    if (!(copy_to_gray_buffer())) {
      /* find star position */
      get_stars();

      /* calc center of gravity */
      /* check rects are set */
      for (i=0; i<2; i++) {
	search_peak(i);
	calc_cg(i);
      }

      /* check stars are found */
      if (mseemon.peakval[0] > mseemon.min_peak &&
	  mseemon.peakval[1] > mseemon.min_peak) {
	/* stars seem to be in */
	/* follow cg */
	mseemon.star_center[0].cx = (int)(mseemon.cg[0].x+0.5);
	mseemon.star_center[0].cy = (int)(mseemon.cg[0].y+0.5);
	mseemon.star_center[1].cx = (int)(mseemon.cg[1].x+0.5);
	mseemon.star_center[1].cy = (int)(mseemon.cg[1].y+0.5);
	
	redraw_image();

	/* display */
	sprintf(str, "guiding. x1:%5.1f y1:%5.1f x2:%5.1f y2:%5.1f",
		mseemon.cg[0].x, mseemon.cg[0].y,
		mseemon.cg[1].x, mseemon.cg[1].y);
	gtk_entry_set_text(GTK_ENTRY(ent_r0), str);
      } else {
	printf("seems to lost star(s)..\n");

	ret = recover_stars(); /* in telctl_automatic.c */
	
	if (ret == EXIT_FAILURE) {
	  printf("turning off guide.\n");
	  if (fout != NULL) {
	    fprintf(fout, "# guiding seems to have failed: lost star(s)?\n");
	    fprintf(fout, "# turning off guide.\n");
	  }
	  gtk_entry_set_text(GTK_ENTRY(ent_guidemode), "OFF");
	  mseemon.is_guiding = 0;

	  /* reset (in cap_window_callbacks.c) */
	  reset_starfinding();
	  
	  /* if automatic mode and not cloudy, 
	   *  do pointing again */
	  if (mseemon.opmode == 1) {
	    ret = is_cloudy();
	    if (ret == 0) {
	      /* restart from waiting star fixed in finder */
	      mseemon.is_pointing = 3;
	    }
	  }
	}
      }
    }
  }
}


/* callback for timeout reload */
/* v.0.3.1: simplified (skip check_cg) */
void timeout_reload_image(gpointer pointer)
{
  int ret, val;

  #ifdef DEBUG
  printf("# tel_stat:%d is_pointing:%d is_guiding:%d\n",
	 mseemon.tel_stat, mseemon.is_pointing, 
	 mseemon.is_guiding);
  #endif
  /* check previous reload is finished */
  if (mseemon.flg_reload) {
    printf("### WARNING: timer overflow\n");
    return;
  }

  /* turn on reloading flg */
  mseemon.flg_reload = 1;

  /* pointing mode ? 
   * v.0.5.0 2003.09.10 
   */
  if (mseemon.is_pointing > 0) {
    /* check whether finder target pos. is set */
    if (mseemon.finder_targetpos.cx < -500) {
      mydialog("please set finder target position.");
      mseemon.is_pointing = 0;
      mseemon.opmode = 0;
      gtk_entry_set_text(GTK_ENTRY(ent_opmode), "Manual");
      mseemon.is_capturing = 1;
      mseemon.is_tracking = 0;
      mseemon.is_guiding = 0;
      gtk_button_set_label(GTK_BUTTON(btn1), "Start Tracking");
      gtk_button_set_label(GTK_BUTTON(btn2), "Start Analysis");
      gtk_button_set_label(GTK_BUTTON(btn3), "Start Recording");
      set_lamp(g_lamp_area1, 0);
      set_lamp(g_lamp_area2, 0);
      set_lamp(g_lamp_area3, 0);
    } else {
      ret = pointing_proc(); /* in telctl_automatic.c */
      if (ret == EXIT_FAILURE) { /* check result. added v0.5.5 20040217 */
	printf("# pointing_proc() failed. trying pointing again...\n");
	if (fout != NULL) {
	  fprintf(fout, "# pointing_proc(): trying pointing again...\n");
	}
	mseemon.is_pointing = 1;
      }
    }
  }  else {
    /* is guiding on?
     * v.0.4.1 2003.05.28
     */
    if (mseemon.is_guiding) {
      #ifdef DEBUG
      printf("guiding mode\n");
      #endif
      reload_image_guiding(&pointer);
    } else {
      if (mseemon.is_capturing == 1) {
	/* not analysis mode; quick preview */
	reload_image_quick(&pointer);
      } else {
	/* analysis mode */
	reload_image(&pointer);
      }
    
      /* check if guiding needed
       * added v.0.4.1 */
      if (mseemon.is_tracking && mseemon.tel_stat > 0 && mseemon.guideflg) {
	check_guid();
      }
    }
  }

  if (mseemon.opmode == 1 && mseemon.is_sleeping == 0) {
    /* check automatic mode status */
    check_auto_status(); /* in telctl_automatic.c */

    get_peak_average(); /* in cap_window.c */
    #ifdef DEBUG
    printf("# number of average data:%d. ave.peak=%d. \n", mseemon.n_average, 
mseemon.peak_average);
    #endif
    if (mseemon.peak_average < 1) mseemon.peak_average=1;
    if (mseemon.n_average >= NAVERAGE) {
      /* check peak average and 
       * change IICCD shutter speed if needed */
      if (mseemon.peak_average < LOW_PEAK || mseemon.peak_average > HIGH_PEAK) {
	val = (int)(mseemon.time_iishutter * OPT_PEAK / mseemon.peak_average);
	if (val > 9999) {
	  val = 9999;
	}
	if (val < 1000) {
	  val = 1000;
	}
	if (mseemon.time_iishutter != val) {
	  #ifdef DEBUG
	  printf("# ave.peak=%d. change IICCD %d to %dusec.\n", mseemon.peak_average, mseemon.time_iishutter, val);
	  #endif
	  if (fout != NULL) {
	    fprintf(fout, "# ave.peak=%d. change IICCD %d to %dusec.\n", mseemon.peak_average, mseemon.time_iishutter, val);
	  }
	  set_iishutter(val); /* in telctl_automatic.c */
	  reset_peak_history(); /* in cap_window.c */
	}
      }
	
      /* if peak_average is high enough, adjust shutter speed 
	 if possible 
	 added v0.7.0 20050128 */
      if (mseemon.time_iishutter > 6666 && mseemon.peak_average > OPT_PEAK) {
	val = (int)(mseemon.time_iishutter * OPT_PEAK * 0.7 / mseemon.peak_average);
	if (val > 9999) {
	  val = 9999;
	}
	if (val < 1000) {
	  val = 1000;
	}
	if (mseemon.time_iishutter != val) {
	  #ifdef DEBUG
	  printf("## ave.peak=%d. change IICCD %d to %dusec.\n", mseemon.peak_average, mseemon.time_iishutter, val);
	  #endif
	  if (fout != NULL) {
	    fprintf(fout, "## ave.peak=%d. change IICCD %d to %dusec.\n", mseemon.peak_average, mseemon.time_iishutter, val);
	  }
	  set_iishutter(val); /* in telctl_automatic.c */
	  reset_peak_history(); /* in cap_window.c */
	}
      }
    }
  }

  /* turn off reloading flg */
  mseemon.flg_reload = 0;

  /* update Main Window text entries */
  update_main_displays();
   
  /* somehow, we should print out something */
  if (mseemon.is_guiding) {
    printf("guiding mode: time out process ended.\n");
  } else {
    printf("current FWHMc: %5.2f %5.2f\n", mseemon.fwhm_corr.x, mseemon.fwhm_corr.y);
  }


}


/* output resultant r0 and FWHM */
void display_r0 (void) {
  gchar str[100];
  //  int el;
  //  float ha;
  double d, tmjd, lst;
  float alt, min_alt, ha;
  myTime mt;

  if (mseemon.selnum >= 0) {
    /* update ctime */
    get_localtime(&(mseemon.ctime));
    get_time(&mt);
    d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
    tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
    lst = get_lst(tmjd, msite.slon);
    get_azel(mseemon.selnum, lst);
    alt = (float)(obj_edb[mseemon.selnum]->s_alt * 180. / PI);
    ha = (float)(obj_edb[mseemon.selnum]->s_ha * 12./PI);
  } else {
    fetchTelRA();
    fetchTelDec();
    alt = mseemon.tel_el * 180. / PI;
    ha = (float)(mseemon.tel_ha * 12. / PI);
  }

  //  sprintf(str, "peak:%3d %3d r0:tran(x):%5.2f lon(y):%5.2f FWHM:%5.2f %5.2f NData:%d/%d el:%4.1f FWHMcorr: %5.2f %5.2f",
  //	  mseemon.peakval[0], mseemon.peakval[1],
  //	  mseemon.r0.x, mseemon.r0.y,
  //	  mseemon.fwhm.x, mseemon.fwhm.y, Ndata, n_calc_r0, 
  //	  (mseemon.tel_el*180./PI), mseemon.fwhm_corr.x, mseemon.fwhm_corr.y);
  sprintf(str, "peak:%3d %3d r0:tran(x):%5.2f lon(y):%5.2f FWHM:%5.2f %5.2f NData:%d/%d el:%4.1f HA:%4.1f FWHMc:%5.2f %5.2f",
	  mseemon.peakval[0], mseemon.peakval[1],
	  mseemon.r0.x, mseemon.r0.y,
	  mseemon.fwhm.x, mseemon.fwhm.y, Ndata, n_calc_r0, 
	  alt, ha, mseemon.fwhm_corr.x, mseemon.fwhm_corr.y);

  gtk_entry_set_text(GTK_ENTRY(ent_r0), str);
  
}

/* reset averaged peak values
 * v0.6.2 20041127
 */
void reset_peak_history(void) {
  int i;
  for (i = 0; i < NAVERAGE; i++) {
    mseemon.peakhistory[i] = 0;
  }
  mseemon.n_average = 0;
  mseemon.peak_average = 0;
}

/* get average */
int get_peak_average(void) {
  int i, val;
  
  if (mseemon.n_average < NAVERAGE) {
    mseemon.peakhistory[mseemon.n_average] = (mseemon.peakval[0] + mseemon.peakval[1])/2;
    mseemon.n_average++;
    val = 0;
    for (i = 0; i < mseemon.n_average; i++) {
      val += mseemon.peakhistory[i];
    }
    mseemon.peak_average = val / mseemon.n_average;
  } else {
    val = 0;
    for (i = 1; i < NAVERAGE; i++) {
      mseemon.peakhistory[i-1] = mseemon.peakhistory[i];
      val += mseemon.peakhistory[i-1];
    }
    mseemon.peakhistory[NAVERAGE-1] = (mseemon.peakval[0] + mseemon.peakval[1])/2;
    val += mseemon.peakhistory[NAVERAGE-1];
    mseemon.peak_average = val / NAVERAGE;
  }

  return(mseemon.peak_average);
}

/* start timer */
void starttimer(void) {

  timertag = gtk_timeout_add(mseemon.time_reload, (GtkFunction)timeout_reload_image, g_image_area1);
  if (fout != NULL) {
    fprintf(fout, "# starttimer() called. timertag is %d.\n", timertag);
  }
  #ifdef DEBUG
  printf("# starttimer() called. timertag is %d.\n", timertag);
  #endif
}

/* stop timer */
void stoptimer(void) {
#ifdef DEBUG
  printf("stoptimer is called.timertag=%d\n",timertag);
#endif
  if (fout != NULL) {
    fprintf(fout, "# stoptimer is called.timertag=%d\n",timertag);
  }
  if (timertag > 0) {
    gtk_timeout_remove(timertag);
    timertag = -1;
    unset_current_seeing(); /* in outputlog.c */
  }
}

/* initialization of capture window 
 *
 */
int init_cap_window(GtkWidget *window_cap) {
  int i;
  GtkWidget *button;
  GtkWidget *box_cap1, *box_cap2;

  box_cap1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window_cap), box_cap1);

  /* load pix buffer and 
   * know the width and height of the image */
  pixbuf = NULL;
  if ((load_pix_buffer())) {
    printf("error in reading from capture image.\n");
    return(EXIT_FAILURE);
  }
  if ((graybuf = (guchar*)malloc(sizeof(unsigned char)*wid*hgt)) == NULL) {
    printf("failed in allocating memory for image buffer.\n");
    //    exit(EXIT_FAILURE);
    // modified 20041209 v0.6.5 iwata
    return(EXIT_FAILURE);
  }

  /* drawing area for capture window */
  gdk_rgb_init(); /* init of GdkRGB */
  g_image_area1 = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(g_image_area1), wid, hgt);
  gtk_signal_connect(GTK_OBJECT(g_image_area1), "expose-event", 
		     GTK_SIGNAL_FUNC(on_darea_expose), NULL);
  gtk_signal_connect(GTK_OBJECT(g_image_area1), "button_press_event", 
		     GTK_SIGNAL_FUNC(on_click_darea), NULL);
  gtk_widget_set_events(g_image_area1, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);

  /* prepare rect. buffers */
  for (i=0; i < 2; i++) {
    if ((alloc_rect(i, mseemon.star_rect[i].cx, mseemon.star_rect[i].cy)) == EXIT_FAILURE) {
      printf("error occured during memory allocation.\n");
      return(EXIT_FAILURE);
    }
  }
    
  #ifdef DEBUG
  printf("window_cap: %p\n", window_cap);
  printf("image_area: %p\n", g_image_area1);
  printf("pImRect0: %p, 1: %p\n", pImRect[0], pImRect[1]);
  #endif

  gtk_box_pack_start(GTK_BOX(box_cap1), g_image_area1, TRUE, TRUE, 0);
  gtk_widget_show(g_image_area1);

  /* output r0 and FWHM window */
  box_cap2 = gtk_hbox_new (FALSE, 0);
  ent_r0 = gtk_entry_new();
  gtk_entry_set_width_chars(GTK_ENTRY(ent_r0), 70);
  gtk_box_pack_start(GTK_BOX(box_cap2), ent_r0, TRUE, TRUE, 0);
  gtk_widget_show(ent_r0);
  gtk_box_pack_start(GTK_BOX(box_cap1), box_cap2, TRUE, TRUE, 0);
  gtk_widget_show (box_cap2);

  box_cap2 = gtk_hbox_new (FALSE, 0);

  /* status color bars */
  g_lamp_area1 = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(g_lamp_area1), (wid/3), 20);
  gtk_signal_connect(GTK_OBJECT(g_lamp_area1), "expose-event", 
		     GTK_SIGNAL_FUNC(on_lamp_expose), NULL);
  gtk_widget_set_events(g_lamp_area1, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
  gtk_box_pack_start(GTK_BOX(box_cap2), g_lamp_area1, TRUE, TRUE, 0);
  gtk_widget_show (g_lamp_area1);

  g_lamp_area2 = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(g_lamp_area2), (wid/3), 20);
  gtk_signal_connect(GTK_OBJECT(g_lamp_area2), "expose-event", 
		     GTK_SIGNAL_FUNC(on_lamp_expose), NULL);
  gtk_widget_set_events(g_lamp_area2, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
  gtk_box_pack_start(GTK_BOX(box_cap2), g_lamp_area2, TRUE, TRUE, 0);
  gtk_widget_show (g_lamp_area2);

  g_lamp_area3 = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(g_lamp_area3), (wid/3), 20);
  gtk_signal_connect(GTK_OBJECT(g_lamp_area3), "expose-event", 
		     GTK_SIGNAL_FUNC(on_lamp_expose), NULL);
  gtk_widget_set_events(g_lamp_area3, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
  gtk_box_pack_start(GTK_BOX(box_cap2), g_lamp_area3, TRUE, TRUE, 0);
  gtk_widget_show (g_lamp_area3);

  gtk_box_pack_start(GTK_BOX(box_cap1), box_cap2, TRUE, TRUE, 0);
  gtk_widget_show (box_cap2);

  box_cap2 = gtk_hbox_new (FALSE, 0);

  /* track button */
  button = gtk_button_new_with_label ("Start Tracking");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_track_button), (gpointer)button);
  gtk_box_pack_start(GTK_BOX(box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  btn1 = button;

  /* analysis button */
  button = gtk_button_new_with_label ("Start Analysis");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_ana_button), (gpointer)button);
  gtk_box_pack_start(GTK_BOX(box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  btn2 = button;

  /* record button */
  button = gtk_button_new_with_label ("Start Recording");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_record_button), (gpointer)button);
  gtk_box_pack_start(GTK_BOX(box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  btn3 = button;

  gtk_box_pack_start(GTK_BOX(box_cap1), box_cap2, TRUE, TRUE, 0);
  gtk_widget_show (box_cap2);

  box_cap2 = gtk_hbox_new (FALSE, 0);

  /* button for setting current finder pos as target
   * added v0.4.8 20030909 iwata */
  button = gtk_button_new_with_label ("Set Currnet Finder Pos. as Target");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_set_targetpos), (gpointer)button);
  gtk_box_pack_start(GTK_BOX(box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  /* button for pointing and centering 
   * added v0.5.2 20031002 */
  button = gtk_button_new_with_label("Point and Center");
  g_signal_connect(G_OBJECT (button), "clicked",
		   G_CALLBACK (callback_point_and_center), (gpointer)button);
  gtk_box_pack_start(GTK_BOX(box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  gtk_box_pack_start(GTK_BOX(box_cap1), box_cap2, TRUE, TRUE, 0);
  gtk_widget_show (box_cap2);

  box_cap2 = gtk_hbox_new (FALSE, 0);

  /* find rect. button */
  button = gtk_button_new_with_label ("Find Star Pos.");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_reset_starpos), (gpointer)button);
  gtk_box_pack_start(GTK_BOX(box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  /* reset button */
  /* hidden v.0.5.5 2004.02.18 */
  //  button = gtk_button_new_with_label ("Reset");
  //  g_signal_connect (G_OBJECT (button), "clicked",
  //		    G_CALLBACK (callback_reset), g_image_area1);
  //  gtk_box_pack_start(GTK_BOX (box_cap2), button, TRUE, TRUE, 0);
  //  gtk_widget_show (button);

  /* stop op. button */
  button = gtk_button_new_with_label ("Stop Op. Now");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_stopop_button), g_image_area1);
  gtk_box_pack_start(GTK_BOX (box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  /* close button for capture window */
  button = gtk_button_new_with_label ("Close");
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK (callback_hide_window_cap), window_cap);


  gtk_box_pack_start(GTK_BOX (box_cap2), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
  gtk_box_pack_start(GTK_BOX(box_cap1), box_cap2, TRUE, TRUE, 0);
  gtk_widget_show (box_cap2);
  gtk_widget_show (box_cap1);


  return(EXIT_SUCCESS);
}

