/* 
 * command from file
 * 2004.11.26-
 * Time-stamp: <2006-01-03 02:20:12 dimm>
 */
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "seemon2.h"

/* start command-check timer */
void start_comm_timer(void) {
  timertag_comm = gtk_timeout_add(mseemon.time_reload_comm, (GtkFunction)timeout_comm_check, NULL);
}

/* stop command-check timer */
void stop_comm_timer(void) {
  gtk_timeout_remove(timertag_comm);
  timertag_comm = -1;
}

/* command check */
/* v0.6.8 update: for long sleep */
/* v0.7.6 update: try auto-mode 3 times
 *  if failed, execute stop-op process 20051013 */
void timeout_comm_check(gpointer pointer) {
  FILE *fp;
  char buf[256];
  int ret, ntry = 0;
  int comm = 0;

  #ifdef DEBUG
  printf("# timeout_comm_check() called.\n");
  printf("#       opmode: %d\n", mseemon.opmode);
  printf("#    op_ssmode: %d\n", mseemon.op_ssmode);
  printf("# is_capturing: %d\n", mseemon.is_capturing);
  printf("# is_tracking: %d\n", mseemon.is_tracking);
  printf("# is_recording: %d\n", mseemon.is_recording);
  printf("# is_guiding: %d\n", mseemon.is_guiding);
  printf("# is_pointing: %d\n", mseemon.is_pointing);
  printf("# is_sleeping: %d\n", mseemon.is_sleeping);
  printf("# sleep_length: %d\n", mseemon.sleep_length);
  printf("# timeout_comm_check() called.\n");
  if (fout != NULL) {
    fprintf(fout, "#       opmode: %d\n", mseemon.opmode);
    fprintf(fout, "#    op_ssmode: %d\n", mseemon.op_ssmode);
    fprintf(fout, "# is_capturing: %d\n", mseemon.is_capturing);
    fprintf(fout, "#  is_tracking: %d\n", mseemon.is_tracking);
    fprintf(fout, "# is_recording: %d\n", mseemon.is_recording);
    fprintf(fout, "#   is_guiding: %d\n", mseemon.is_guiding);
    fprintf(fout, "#  is_pointing: %d\n", mseemon.is_pointing);
    fprintf(fout, "#  is_sleeping: %d\n", mseemon.is_sleeping);
    fprintf(fout, "# sleep_length: %d\n", mseemon.sleep_length);
  }
  #endif

  if (mseemon.is_pointing > 0) {
    /* comment out v0.7.0 20050128 */
    //      mseemon.is_sleeping > 0) {
    check_end_time(); /* in telctl_automatic.c */
  } else {
    if (mseemon.tel_stat == 4 || mseemon.tel_stat == 0) {
      /* make sure timer is stopped */
      stoptimer(); /* in cap_window.c */
    }
  }
   

  if ((fp = fopen(COMMANDFILE, "r")) != NULL) {
    fgets(buf, 256, fp);
    #ifdef DEBUG
    printf("# received command:%s",buf);
    #endif
    if (!(strncmp(buf, "start", 5)) || !(strncmp(buf, "START", 5))) {
      comm = 1;
    }
    if (!(strncmp(buf, "quit", 4)) || !(strncmp(buf, "QUIT", 4))) {
      comm = 2;
    }

    if (!(strncmp(buf, "stop", 4)) || !(strncmp(buf, "STOP", 4))) {
      comm = 3;
    }
    if (!(strncmp(buf, "auto", 4)) || !(strncmp(buf, "AUTO", 4))) {
      comm = 4;
    }
    if (!(strncmp(buf, "pause", 5)) || !(strncmp(buf, "PAUSE", 5))) {
      comm = 5;
    }
    if (!(strncmp(buf, "next", 4)) || !(strncmp(buf, "NEXT", 4))) {
      comm = 6;
    }
    
    fclose(fp);

    sprintf(buf, "/bin/rm -f %s\n", COMMANDFILE);
    #ifdef DEBUG
    printf("# execute %s", buf);
    #endif
    system(buf);
  }

  switch(comm) {
  case 1:
    printf("# timeout_comm_check(): received start command.\n");

    /* intialize log file */
    setoutputfile();

    /* socket connection */
    fd_socket = -1;
    if ((ret = connect_socket(socket_server)) == EXIT_SUCCESS) {
      printf("connected to %s.\n", socket_server);
    } else {
      if (fout != NULL) {
	fprintf(fout, "unable to connect to %s", socket_server);
	fd_socket=-1;
      }
    }
    break;
  case 2:
    printf("# timeout_comm_check(): received quit command.\n");
    stop_comm_timer();
    stop_operation(); /* in telctl_automatic.c */
    break;
  case 3:
    printf("# timeout_comm_check(): received stop command.\n");
    stop_operation(); /* in telctl_automatic.c */
    break;
  case 4:
    printf("# timeout_comm_check(): received auto_start command.\n");
    mseemon.autoflg = 1; /* set auto mode true */
    if (mseemon.is_sleeping > 0) {
      /* set to resume obs immediately */
      mseemon.is_sleeping = mseemon.sleep_length;
    } else {
      /* intialize log file */
      setoutputfile();

      /* socket connection */
      fd_socket = -1;
      if ((ret = connect_socket(socket_server)) == EXIT_SUCCESS) {
	printf("connected to %s.\n", socket_server);
	
	while (ntry < 3) {
	  ret = auto_startup(); /* in telctl_automatic.c */
	  if (ret == EXIT_FAILURE) {
	    ntry++;
	    printf("# wait 30 sec..\n");
	    sleep(30);
	  } else {
	    ntry = 3;
	  }
	}
	if (ret == EXIT_FAILURE) {
	  fprintf(fout,"# failed to start auto-mode operation.\n");
	  fprintf(fout,"# stop operation now...\n");
	  printf("# failed to start auto-mode operation.\n");
	  printf("# stop operation now...\n");
	  stop_operation(); /* in telctl_automatic.c */
	}
      } else {
	if (fout != NULL) {
	  fprintf(fout, "# unable to connect to %s", socket_server);
	  fd_socket=-1;
	}
      }
    }
    break;
  case 5:
    printf("# timeout_comm_check(): received pause command.\n");
    stoptimer();
    reset_starfinding(); /* in cap_window_callbacks.c */
    gtk_widget_hide((GtkWidget*)window_capture);
    mseemon.is_capturing = 0;
    break;
  case 6:
    printf("# timeout_comm_check(): received next command.\n");
    if (mseemon.autoflg) {
      obj_edb[mseemon.selnum]->o_flags = 1; /* flag current object */
      auto_startup(); /* in telctl_automatic.c */
    }
    break;
  }

  /* long sleep ? */
  if (mseemon.is_sleeping > 0) {
    mseemon.is_sleeping++;
    if (timertag != -1) {
      stoptimer(); /* in cap_window.c */
    }

    printf("# long sleep: %d, %d left.\n", mseemon.is_sleeping, mseemon.sleep_length - mseemon.is_sleeping);

    if (fout != NULL) {
      fprintf(fout, "# long sleep: %d min.\n", mseemon.is_sleeping);
      #ifdef DEBUG
      printf("# long sleep: %d min. ", mseemon.is_sleeping);
      printf("# is_pointing=%d.\n", mseemon.is_pointing);
      #endif
      if (mseemon.is_sleeping >= mseemon.sleep_length) {
	mseemon.is_sleeping = 0;
	#ifdef DEBUG
	printf("# woke up from long sleep.\n");
        #endif
	if (fout != NULL) {
	  fprintf(fout,"# woke up from long sleep.\n");
	}
	resume_obs(); /* in telctl_automatic.c */
      }
    }
  }
 
  printf("# finished comm_check.\n");
}

