/*
 * get_cg.c : calc center of gravity and r0
 *
 * based on seemon1 GetCG.cpp
 *
 * 2003.01.27 -
 * Time-stamp: <2005-12-13 15:10:48 dimm>
 */

#include <stdlib.h>
#include <math.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "seemon2.h"

/* allocate memory for pImRect */
int alloc_rect(int n, int tx, int ty) {
  /* if already exist, free them */
  if (pImRect[n] != NULL) {
    free(pImRect[n]);
  }

  if ((pImRect[n] = (int*)malloc(sizeof(int) * tx * ty)) == NULL) {
    printf("ERROR: failed in memory allocation for rect. image buffer.\n");
    return(EXIT_FAILURE);
  }

  return(EXIT_SUCCESS);
}


/**********************************
 * estimate background (light O version)
 * 2002.02.09 slightly modified
 * 2003.01.27 adopted for seemon2
 **********************************/
int GetBackground(int n) {
  int i, t, t2, num = 0, sum = 0, sum2 = 0;
  double sigma;

  int len = mseemon.star_rect[n].cx * mseemon.star_rect[n].cy;

  for (i = 0; i < len; i++) {
    sum += pImRect[n][i];
    sum2 += pImRect[n][i]*pImRect[n][i];
    num++;
  }

  if (num > 0) {
    t = sum / num;
    t2 = sum2 / num;
    sigma = sqrt(t2 - t*t);

    /* calc mean along with sigma clipping */
    sum = 0;
    num = 0;
    for (i = 0; i < len; i++) {
      //      if (pImRect[n][i] < mseemon.skyval + (int)(bgthr * sigma) &&
      //	  pImRect[n][i] > mseemon.skyval - (int)(bgthr * sigma)) {
      if (pImRect[n][i] <  (int)(bgthr * sigma) &&
	  pImRect[n][i] > -(int)(bgthr * sigma)) {
	sum += pImRect[n][i];
	num++;
      }
    }
    if (num > 0) {
      t = sum / num;
    } else {
      t = 0;
    }
  } else {
    t = 0;
  }

  return t;
}


/*********************************
 * search_peak
 * 2003.05.02
 *********************************/
int search_peak(int n)
{
  int max, maxx, maxy;
  int smooth, smhalf, sum;
  int i, j, rx, ry;

  //#ifdef DEBUG
  //  printf("search_peak called. for star %d\n", n);
  //#endif

  smooth = SMOOTH_SIZE;
  smhalf = (int)((smooth -1)/2);

  /* check graybuf */
  if (graybuf == NULL) {
    return(-1);
  }

  /* check rects */
  if (pImRect[n] == NULL) {
    return(-1);
  }

  max = 0;

  for ( i = smhalf; i < mseemon.star_rect[n].cy - smhalf; i++ ) {
    for ( j = smhalf; j < mseemon.star_rect[n].cx - smhalf; j++ ) {
      rx = mseemon.star_center[n].cx + j - mseemon.star_rect[n].cx / 2;
      ry = mseemon.star_center[n].cy + i - mseemon.star_rect[n].cy / 2;
      if (ry < hgt-1 && ry >= 1 && rx < wid-1 && rx >= 1) {
	/* smoothed pixel count */
	sum = graybuf[ (ry-1) * wid + rx - 1] 
	  + graybuf[ (ry -1) * wid + rx]
	  + graybuf[ (ry -1) * wid + rx + 1]
	  + graybuf[ ry * wid + rx - 1]
	  + graybuf[ ry * wid + rx]
	  + graybuf[ ry * wid + rx + 1]
	  + graybuf[ (ry + 1) * wid + rx - 1]
	  + graybuf[ (ry + 1) * wid + rx]
	  + graybuf[ (ry + 1) * wid + rx + 1];
	if (sum > max) {
	  max = sum;
	  maxx = rx;
	  maxy = ry;
	}
      }
    }
  }

  mseemon.peak[n].cx = maxx;
  mseemon.peak[n].cy = maxy;
}


/*********************************
 * calc_cg : calc center of gravity
 *   n: star number (0 or 1)
 *   m: image number (=1) --> omitted
 * 2002.02.09 slightliy modified
 * 2003.01.28 adopted for seemon2
 * 2003.02.16 insert cg motion check (moved from calc_r0)
 * 2003.05.08 ver.0.2.6: calc cg within cg_rect (smaller size)
 */
int calc_cg(int n)
{
  int i, j, rx, ry, tx, ty;
  int tmpx = 0, tmpy = 0;
  int sumintensity = 0;

  /* check graybuf */
  if (graybuf == NULL) {
    return(-1);
  }

  /* check rects */
  if (pImRect[n] == NULL) {
    return(-1);
  }

  /* copy buffer value to pImRect */
  for ( i = 0; i < mseemon.cg_rect[n].cy; i++ ) {
    for ( j = 0; j < mseemon.cg_rect[n].cx; j++ ) {
      rx = mseemon.peak[n].cx + j - mseemon.cg_rect[n].cx / 2;
      ry = mseemon.peak[n].cy + i - mseemon.cg_rect[n].cy / 2;
      if (ry < hgt && ry >= 0 && rx < wid && rx >= 0) {
	pImRect[n][i*mseemon.cg_rect[n].cx + j]  = graybuf[ ry * wid + rx ];
      } else {
	pImRect[n][i*mseemon.cg_rect[n].cx + j]  = 0;
      }
    }
  }

  /* check setting-sky counter */
  //  if (mseemon.cnt_sky == 0) {
    //    #ifdef DEBUG
    //    printf("GetBackground is called.\n");
    //    #endif
    /* get background value with sigma-clipping */
    //    mseemon.skyval = GetBackground(n);
  //    mseemon.skyval = 0;
  //  }
  //  mseemon.cnt_sky++;
  //  if (mseemon.cnt_sky >= mseemon.time_sky) {
  //    mseemon.cnt_sky = 0;
  //  }

  for ( i = 0; i < mseemon.cg_rect[n].cy; i++ ) {
    for ( j = 0; j < mseemon.cg_rect[n].cx; j++ ) {
      //      pImRect[n][i*mseemon.cg_rect[n].cx + j] -= mseemon.skyval;
      tx = j - mseemon.cg_rect[n].cx/2;
      ty = i - mseemon.cg_rect[n].cy/2;
      tmpx += tx * pImRect[n][i*(mseemon.cg_rect[n].cx) + j];
      tmpy += ty * pImRect[n][i*(mseemon.cg_rect[n].cx) + j];
      sumintensity += pImRect[n][i*(mseemon.cg_rect[n].cx) + j];
    }
  }

  if (sumintensity != 0) {
    mseemon.oldcg[n].x = mseemon.cg[n].x;
    mseemon.oldcg[n].y = mseemon.cg[n].y;

    mseemon.cg[n].x = (double)mseemon.peak[n].cx + (double)tmpx * cgfactor / sumintensity;
    mseemon.cg[n].y = (double)mseemon.peak[n].cy + (double)tmpy * cgfactor / sumintensity;
  }

  /* set cg counts */
  tx = (int)(mseemon.cg[n].x+0.5);
  ty = (int)(mseemon.cg[n].y+0.5);
  //  #ifdef DEBUG
  //  printf("tx:%d ty:%d\n", tx, ty);
  //  #endif
  if (tx > 0 && tx < wid && ty > 0 && ty < hgt) {
    mseemon.peakval[n] = (int)graybuf[ty*wid + tx];
    mseemon.starflux[n] = sumintensity;
  }

  return TRUE;
}


/* check cg 
 * 2003.05.08
 * 2003.09.12 simplified: just check image distance
 *            returns 1 if distance is large
 * 2003.10.01 v0.5.1. slight change around IMDIST
 * 2004.08.02 v0.5.10 n_total_sample --> Ndata (iwata)
 */
int check_cg(void) {
  int i, flg = 0;
  dcoord diffpos[2];
  double dx, dy, imdist;

  dx = mseemon.cg[0].x - mseemon.cg[1].x;
  dy = mseemon.cg[0].y - mseemon.cg[1].y;

  /* check cg diff 
   * (skip if too large motion) */
  imdist = sqrt(dx * dx + dy * dy);
  mseemon.image_dist = imdist;

  #ifdef DEBUG
  printf("imdist: %6.2f MAX:%6.2f MIN:%6.2f\n", imdist, IMDIST_MAX, IMDIST_MIN);
  #endif
  if (Ndata > 10) {
    if (imdist > IMDIST_MAX || imdist < IMDIST_MIN) {
      flg++;
    }
  }

  return(flg);

}

/* output to logfile
 * v.0.4.1 2003.05.28
 * v.0.7.8 2005.12.12 added current seeing output
 */
void appendlog (void) {
  char str[20];
  double d, tmjd, lst;
  float alt, min_alt, ha;
  myTime mt;
  
  /* update ctime */
  get_localtime(&(mseemon.ctime));
  get_time(&mt);
  d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
  tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
  lst = get_lst(tmjd, msite.slon);
  get_azel(mseemon.selnum, lst);
  alt = (float)(obj_edb[mseemon.selnum]->s_alt * 180. / PI);
  ha = (float)(obj_edb[mseemon.selnum]->s_ha * 12./PI);

  if (mseemon.fwhm.x > 10. ||
      mseemon.fwhm.y > 10. ||
      mseemon.fwhm.x < 0.1 ||
      mseemon.fwhm.y < 0.1) {
    sprintf(str, "#%04d.%02d.%02d-%02d:%02d:%02d", 
	    mseemon.ctime.year, 
	    mseemon.ctime.month,
	    mseemon.ctime.day,
	    mseemon.ctime.hour,
	    mseemon.ctime.min,
	    mseemon.ctime.sec);
  } else {
    sprintf(str, " %04d.%02d.%02d-%02d:%02d:%02d", 
	    mseemon.ctime.year, 
	    mseemon.ctime.month,
	    mseemon.ctime.day,
	    mseemon.ctime.hour,
	    mseemon.ctime.min,
	    mseemon.ctime.sec);
  }

  if (fout != NULL) {
    fprintf(fout, "%s %6.2f %6.2f %6.2f %6.2f %11.4e %11.4e %8.4f %8.4f %6.3f %6.3f %6.3f %6.3f %2d/%2d  %5.2f %5.2f %5.2f %5.2f %3d %3d %d %d %4d ", 
	    str, 
	    mseemon.cg[0].x, mseemon.cg[0].y,
	    mseemon.cg[1].x, mseemon.cg[1].y,
	    mseemon.diffcg[n_calc_r0-1].x, mseemon.diffcg[n_calc_r0-1].y,
	    mseemon.r0.x, mseemon.r0.y, 
	    mseemon.fwhm.x, mseemon.fwhm.y,
	    mseemon.fwhm_corr.x, mseemon.fwhm_corr.y,
	    Ndata, n_calc_r0,
	    (mseemon.tel_ra * 180. / PI), (mseemon.tel_dc * 180. / PI),
	    ha, alt, 
	    mseemon.peakval[0], mseemon.peakval[1],
	    mseemon.starflux[0], mseemon.starflux[1],
	    mseemon.time_iishutter);

    /* check peak saturation 
     * added v0.5.7 20040221 */
    if (mseemon.peakval[0] > PEAK_MAX &&
	mseemon.peakval[1] > PEAK_MAX) {
      fprintf(fout, "SATURATION");
    }

    fprintf(fout, "\n");

    /* flush stream; added 0.5.5 20040217 */
    fflush(fout);
  }

  put_current_seeing(mseemon.fwhm_corr); /* in outputlog.c */
}

/* calc r0 */
/* update 2003.03.11: calc corrected FWHM */
/* 2003.05.08 ver.0.2.6 sigma clipping 
 * 2003.05.09 ver.0.2.7 percentile clip
 * 2003.05.09 ver.0.2.8 sigma clipping again
 * 2003.05.16 ver.0.3.1 simplified
 *
 * currently this func. only used in 
 *  quick mode (not in ana. mode)
 */
int calc_r0(void) {
  int i, j;
  double el, zd;
  double fwhm1, fwhm2;
  double dx, dy;
  dcoord Sum, Sigma2, th;
  dcoord DiffCG[N_CALC_R0];

  Sum.x = 0.;
  Sum.y = 0.;

  mseemon.diffcg[n_calc_r0 -1].x = (mseemon.cg[0].x - mseemon.cg[1].x) * scalex;
  mseemon.diffcg[n_calc_r0 -1].y = (mseemon.cg[0].y - mseemon.cg[1].y) * scaley;

  /* clip noise and construct sample for statistics */
  Ndata = 0;
  if (n_calc_r0 >= NSample) {
    for (i = 0; i < n_calc_r0; i++) {
      dx = mseemon.diffcg[i].x - Mean.x;
      dy = mseemon.diffcg[i].y - Mean.y;
      dx = (dx > 0) ? dx: -1.*dx;
      dy = (dy > 0) ? dy: -1.*dy;
      //      #ifdef DEBUG
      //      printf("#############\n");
      //      printf("%e %e %e %e\n", dx, dy, Sigma.x, Sigma.y);
      //      #endif
      if (dx < mseemon.clip * Sigma.x && dy < mseemon.clip * Sigma.y) {
	DiffCG[Ndata].x = mseemon.diffcg[i].x;
	DiffCG[Ndata].y = mseemon.diffcg[i].y;
	Ndata++;
      }
    }
  } else {
    /* not enough data points; copy */
    for (i = 0; i < n_calc_r0; i++) {
      DiffCG[Ndata].x = mseemon.diffcg[i].x;
      DiffCG[Ndata].y = mseemon.diffcg[i].y;
      Ndata++;
    }
  }

  /* Statistics */
  for (i = 0; i < Ndata; i++ ) {
    Sum.x += DiffCG[i].x;
    Sum.y += DiffCG[i].y;
  }

  if (Ndata > 0) {
    Mean.x = Sum.x / Ndata;
    Mean.y = Sum.y / Ndata;
    Sigma2.x = 0.;
    Sigma2.y = 0.;

    for (i = 0; i < Ndata; i++ ) {
      Sigma2.x += DiffCG[i].x * DiffCG[i].x;
      Sigma2.y += DiffCG[i].y * DiffCG[i].y;
    }

    Sigma2.x /= Ndata;
    Sigma2.y /= Ndata;
    Sigma2.x -= Mean.x*Mean.x;
    Sigma2.y -= Mean.y*Mean.y;

    /* x=longitudinal, y=transverse. */
    /* 2003.05.15 */
    /* convert to cm */
    /* Sigma2.y: longitudinal, Sigma2.x: transverse */
    /* r0.x: transverse r0.y: longitudinal 2003.03.11 */
    mseemon.r0.x = 100.*pow(r0k.x/Sigma2.x, 3./5.);
    mseemon.r0.y = 100.*pow(r0k.y/Sigma2.y, 3./5.);

    /* calc fwhm */
    mseemon.fwhm.x = 0.98 * Lambda *100./ mseemon.r0.x * 180.*3600./PI;
    mseemon.fwhm.y = 0.98 * Lambda *100./ mseemon.r0.y * 180.*3600./PI;
  } else {
    Mean.x = 0.;
    Mean.y = 0.;
    Sigma2.x = 0.;
    Sigma2.y = 0.;
    mseemon.r0.x = 0.;
    mseemon.r0.y = 0.;
    mseemon.fwhm.x = 0.;
    mseemon.fwhm.y = 0.;
  }

  if (n_calc_r0 < NSample) {
    /* increase n_calc_r0 if smaller than NSample */
    n_calc_r0++;
    /* set Sigma */
    Sigma.x = sqrt(Sigma2.x);
    Sigma.y = sqrt(Sigma2.y);
  } else {
    /* shift */
    for (i = 0; i < n_calc_r0 -1; i++) {
      mseemon.diffcg[i].x = mseemon.diffcg[i+1].x;
      mseemon.diffcg[i].y = mseemon.diffcg[i+1].y;
    }
    /* set Sigma */
    Sigma.x = sqrt(Sigma2.x);
    Sigma.y = sqrt(Sigma2.y);
  }

  /* calc corrected FWHM if telescope is connected */
  if (mseemon.tel_stat >= 1) {
    /* connected */
    fetchTelRA();
    fetchTelDec();
    get_alt(mseemon.tel_ra, mseemon.tel_dc, &(mseemon.tel_el), &(mseemon.tel_ha));
    //    #ifdef DEBUG
    //    printf("current el: %12.5e\n", (mseemon.tel_el)*180./PI);
    //    #endif
    /* zenith distance in radian */
    zd = (PI/2. - mseemon.tel_el);
    mseemon.fwhm_corr.x = mseemon.fwhm.x * pow(cos(zd), 0.6);
    mseemon.fwhm_corr.y = mseemon.fwhm.y * pow(cos(zd), 0.6);
  } else {
    mseemon.fwhm_corr.x = 0.;
    mseemon.fwhm_corr.y = 0.;
    mseemon.tel_el = 0.;
    mseemon.tel_ha = 0.;
  }

  /* check saturation */

  if (mseemon.is_recording) {
    appendlog();
  }

  return(1);
}


/* calc_r0_new
 * 2003.03.11: calc corrected FWHM
 * 2003.05.08 ver.0.2.6 sigma clipping 
 * 2003.05.09 ver.0.2.7 percentile clip
 * 2003.05.09 ver.0.2.8 sigma clipping again
 * 2003.05.16 ver.0.3.1 simplified
 * 2003.10.01 ver.0.5.1 image rotation algorythm 
 *             implemented.
 *             removed sigma clipping
 *             (to reduce overhead)
 *
 * this func. is only used in 
 *  quick mode (not in ana. mode)
 */
int calc_r0_new(void) {
  int i;
  double t, tsum, zd;
  dcoord Sum, Sigma2;
  dcoord DiffCG[N_CALC_R0];
  dcoord v;

  v.x = (mseemon.cg[0].x - mseemon.cg[1].x) * scalex;
  v.y = (mseemon.cg[1].y - mseemon.cg[1].y) * scaley;
  t = v.x/v.y;

  if (n_calc_r0 < NSample) {
    mseemon.image_theta[n_calc_r0] = -1. * atan(t);
    n_calc_r0++;
  } else {
    /* shift */
    for (i=0; i<n_calc_r0-1; i++) {
      mseemon.image_theta[i] = mseemon.image_theta[i+1];
      mseemon.diffcg[i] = mseemon.diffcg[i+1];
    }
    mseemon.image_theta[n_calc_r0] = -1. * atan(t);
  }

  tsum = 0.;
  for (i=0; i<n_calc_r0; i++) {
    tsum += mseemon.image_theta[i];
  }
  tsum /= n_calc_r0;
  mseemon.image_theta0 = tsum;

  #ifdef DEBUG
  printf("# theta: %5.2f\n", tsum * 180./PI);
  #endif

  /* rotation */
  mseemon.diffcg[n_calc_r0-1].x = cos(tsum) * v.x - sin(tsum) * v.y;
  mseemon.diffcg[n_calc_r0-1].y = sin(tsum) * v.x + cos(tsum) * v.y;


  /* Statistics */
  Sum.x = 0.;
  Sum.y = 0.;
  for (i = 0; i < n_calc_r0; i++ ) {
    Sum.x += mseemon.diffcg[i].x;
    Sum.y += mseemon.diffcg[i].y;
  }

  Mean.x = Sum.x / n_calc_r0;
  Mean.y = Sum.y / n_calc_r0;

  Sigma2.x = 0.;
  Sigma2.y = 0.;
  for (i = 0; i < n_calc_r0; i++ ) {
    Sigma2.x += mseemon.diffcg[i].x * mseemon.diffcg[i].x;
    Sigma2.y += mseemon.diffcg[i].y * mseemon.diffcg[i].y;
  }

  Sigma2.x /= n_calc_r0;
  Sigma2.y /= n_calc_r0;
  Sigma2.x -= Mean.x*Mean.x;
  Sigma2.y -= Mean.y*Mean.y;

  /* convert to cm */
  /* Sigma2.y: longitudinal, Sigma2.x: transverse */
  /* r0.x: transverse r0.y: longitudinal 2003.03.11 */
  mseemon.r0.x = 100.*pow(r0k.x/Sigma2.x, 3./5.);
  mseemon.r0.y = 100.*pow(r0k.y/Sigma2.y, 3./5.);

  /* calc fwhm */
  mseemon.fwhm.x = 0.98 * Lambda *100./ mseemon.r0.x * 180.*3600./PI;
  mseemon.fwhm.y = 0.98 * Lambda *100./ mseemon.r0.y * 180.*3600./PI;


  /* calc corrected FWHM if telescope is connected */
  if (mseemon.tel_stat >= 1) {
    /* connected */
    fetchTelRA();
    fetchTelDec();
    get_alt(mseemon.tel_ra, mseemon.tel_dc, &(mseemon.tel_el), &(mseemon.tel_ha));

    /* zenith distance in radian */
    zd = (PI/2. - mseemon.tel_el);
    mseemon.fwhm_corr.x = mseemon.fwhm.x * pow(cos(zd), 0.6);
    mseemon.fwhm_corr.y = mseemon.fwhm.y * pow(cos(zd), 0.6);
  } else {
    mseemon.fwhm_corr.x = 0.;
    mseemon.fwhm_corr.y = 0.;
    mseemon.tel_el = 0.;
    mseemon.tel_ha = 0.;
  }

  if (mseemon.is_recording) {
    appendlog();
  }

  return(1);
}

/* ana_r0 */
/* based on calc_r0,
 * modified for analysis mode
 * 2003.05.17 v.0.4.0u
 * 2004.02.22 v.0.5.7 modified correction of FWHM
 *
 */
int ana_r0(void) {
  int i, j;
  double lst, tmjd, d, zd;
  //  double el, zd;
  double fwhm1, fwhm2;
  double dx, dy;
  myTime mt;
  dcoord Sum, Sigma2, th;
  dcoord DiffCG[N_CALC_R0];

  Sum.x = 0.;
  Sum.y = 0.;

  /* statistics for clipping */
  for (i = 0; i < n_calc_r0; i++) {
    Sum.x += mseemon.diffcg[i].x;
    Sum.y += mseemon.diffcg[i].y;
  }
  Mean.x = Sum.x / n_calc_r0;
  Mean.y = Sum.y / n_calc_r0;

  Sigma2.x = 0.;
  Sigma2.y = 0.;

  for (i = 0; i < n_calc_r0; i++ ) {
    Sigma2.x += mseemon.diffcg[i].x * mseemon.diffcg[i].x;
    Sigma2.y += mseemon.diffcg[i].y * mseemon.diffcg[i].y;
  }

  Sigma2.x /= n_calc_r0;
  Sigma2.y /= n_calc_r0;
  Sigma2.x -= Mean.x*Mean.x;
  Sigma2.y -= Mean.y*Mean.y;
  Sigma.x = sqrt(Sigma2.x);
  Sigma.y = sqrt(Sigma2.y);

  /* clip noise and construct sample for statistics */
  Ndata = 0;
  for (i = 0; i < n_calc_r0; i++) {
    dx = mseemon.diffcg[i].x - Mean.x;
    dy = mseemon.diffcg[i].y - Mean.y;
    dx = (dx > 0) ? dx: -1.*dx;
    dy = (dy > 0) ? dy: -1.*dy;
    if (dx < mseemon.clip * Sigma.x && dy < mseemon.clip * Sigma.y) {
      DiffCG[Ndata].x = mseemon.diffcg[i].x;
      DiffCG[Ndata].y = mseemon.diffcg[i].y;
      Ndata++;
    }
  }

  //#ifdef DEBUG
  //  printf("%d among %d clipped.\n", (n_calc_r0 - Ndata), n_calc_r0);
  //#endif

  Sum.x = 0.;
  Sum.y = 0.;
  /* Statistics for calculation of r0 */
  for (i = 0; i < Ndata; i++ ) {
    Sum.x += DiffCG[i].x;
    Sum.y += DiffCG[i].y;
  }

  if (Ndata > 0) {
    Mean.x = Sum.x / Ndata;
    Mean.y = Sum.y / Ndata;
    Sigma2.x = 0.;
    Sigma2.y = 0.;

    for (i = 0; i < Ndata; i++ ) {
      Sigma2.x += DiffCG[i].x * DiffCG[i].x;
      Sigma2.y += DiffCG[i].y * DiffCG[i].y;
    }

    Sigma2.x /= Ndata;
    Sigma2.y /= Ndata;
    Sigma2.x -= Mean.x*Mean.x;
    Sigma2.y -= Mean.y*Mean.y;

    /* y=longitudinal, x=transverse. */
    /* 2003.05.15 */
    /* convert to cm */
    /* Sigma2.y: longitudinal, Sigma2.x: transverse */
    /* r0.x: transverse r0.y: longitudinal 2003.03.11 */
    mseemon.r0.x = 100.*pow(r0k.x/Sigma2.x, 3./5.);
    mseemon.r0.y = 100.*pow(r0k.y/Sigma2.y, 3./5.);

    /* calc fwhm */
    mseemon.fwhm.x = 0.98 * Lambda *100./ mseemon.r0.x * 180.*3600./PI;
    mseemon.fwhm.y = 0.98 * Lambda *100./ mseemon.r0.y * 180.*3600./PI;
  } else {
    Mean.x = 0.;
    Mean.y = 0.;
    Sigma2.x = 0.;
    Sigma2.y = 0.;
    mseemon.r0.x = 0.;
    mseemon.r0.y = 0.;
    mseemon.fwhm.x = 0.;
    mseemon.fwhm.y = 0.;
  }


  get_time(&mt);
  d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
  tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
  lst = get_lst(tmjd, msite.slon);
  get_azel(mseemon.selnum, lst);

  zd = (PI/2. - obj_edb[mseemon.selnum]->s_alt);
  mseemon.fwhm_corr.x = mseemon.fwhm.x * pow(cos(zd), 0.6);
  mseemon.fwhm_corr.y = mseemon.fwhm.y * pow(cos(zd), 0.6);

  /* calc corrected FWHM if telescope is connected */
  //  if (mseemon.tel_stat >= 1) {
    /* connected */
  //    fetchTelRA();
  //    fetchTelDec();
  //    get_alt(mseemon.tel_ra, mseemon.tel_dc, &(mseemon.tel_el), &(mseemon.tel_ha));
    //    #ifdef DEBUG
    //    printf("current el: %12.5e\n", (mseemon.tel_el)*180./PI);
    //    #endif
    /* zenith distance in radian */
  //    zd = (PI/2. - mseemon.tel_el);
  //    mseemon.fwhm_corr.x = mseemon.fwhm.x * pow(cos(zd), 0.6);
  //    mseemon.fwhm_corr.y = mseemon.fwhm.y * pow(cos(zd), 0.6);
  //  } else {
  //    mseemon.fwhm_corr.x = 0.;
  //    mseemon.fwhm_corr.y = 0.;
  //    mseemon.tel_el = 0.;
  //    mseemon.tel_ha = 0.;
  //  }

  if (mseemon.is_recording) {
    appendlog();
  }

  return(1);
}


/* copy diffcg to prev_diffcg
 * 20041202
 */
void copy_diffcg(int n) {
  int i;
  for (i = 0; i < n; i++) {
    mseemon.prev_diffcg[i].x = mseemon.diffcg[i].x;
    mseemon.prev_diffcg[i].y = mseemon.diffcg[i].y;
  }
}

/* compare prev_diffcg and diffcg
 * return 1 if same, 0 if not */
int cmp_diffcg(int n) {
  int i, flg;
  flg = 1;
  for (i = 0; i < n; i++) {
    if (mseemon.prev_diffcg[i].x != mseemon.diffcg[i].x ||
	mseemon.prev_diffcg[i].y != mseemon.diffcg[i].y)
      flg = 0;
  }
  return flg;
}

