/* codes related to object database file (.edb)
 * Time-stamp: <2004-11-27 15:21:35 dimm>
 *
 * 2002.11.06 created
 * 2002.11.08 merge into gtk/seemon2
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include "seemon2.h"

int read_edb(char* fname) {
  FILE *fp;
  int entnum = 0;
  int flg, iptr, optr;
  int i, h, m, d;
  float s, rad;
  Obj* cobj;
  char buf[256], sbuf[256], tbuf[5];

  if ((fp = fopen(fname, "r")) == NULL) {
    printf("failed in opening %s.\n", fname);
    return(-1);
  }

  while (fgets(buf, 256, fp)) {
    /* allocate memory */
    if ((cobj = (Obj*)malloc(sizeof(Obj))) == NULL) {
      printf("out of memory in reading edb file. %d\n", entnum);
      return(-1);
    }

    flg = 1;
    iptr = 0;
    optr = 0;
    while (flg) {
      if (buf[iptr] == '\0') {
	/* end of line */
	sbuf[optr] = '\0';
	/* epoch */
	cobj->f_epoch = (float)atof(sbuf);
	flg = 0;
      } else if  (buf[iptr] == ',') {
	/* found a separator */
	sbuf[optr] = '\0';
	switch (flg) {
	case 1:
	  strcpy(cobj->o_name, sbuf);
	  break;
	case 2:
	  /* modified v0.6.2 20041127
	   * used to be spectrum type,
	   * changed for pointing flagging
	  cobj->o_flags = sbuf[0];
	   */
	  cobj->o_flags = 0;
	  break;
	case 3: /* ra */
	  optr = 0;
	  /* hour */
	  tbuf[0] = sbuf[optr];
	  optr++;
	  if (sbuf[optr] == ':') {
	    tbuf[1] = '\0';
	  } else {
	    tbuf[1] = sbuf[optr];
	    tbuf[2] = '\0';
	    optr++;
	  }
	  optr++;
	  h = atoi(tbuf);
	  /* min */
	  tbuf[0] = sbuf[optr];
	  optr++;
	  if (sbuf[optr] == ':') {
	    tbuf[1] = '\0';
	  } else {
	    tbuf[1] = sbuf[optr];
	    tbuf[2] = '\0';
	    optr++;
	  }
	  optr++;
	  m = atoi(tbuf);
	  /* sec */
	  for (i = 0; sbuf[optr] != '\0'; i++) {
	    tbuf[i] = sbuf[optr];
	    optr++;
	  }
	  tbuf[i] = '\0';
	  optr++;
	  s = (float)atof(tbuf);
	  rad = (float)((h * 15. + m/4. + s/240.) * PI / 180.);
	  cobj->s_ra = rad;
	  break;
	case 4: /* dec */
	  optr = 0;
	  /* deg */
	  tbuf[0] = sbuf[optr];
	  optr++;
	  if (tbuf[0] == '-') { /* south */
	    tbuf[1] = sbuf[optr];
	    optr++;
	    if (sbuf[optr] == ':') {
	      tbuf[2] = '\0';
	    } else {
	      tbuf[2] = sbuf[optr];
	      tbuf[3] = '\0';
	      optr++;
	    }
	  } else { /* north */
	    if (sbuf[optr] == ':') {
	      tbuf[1] = '\0';
	    } else {
	      tbuf[1] = sbuf[optr];
	      tbuf[2] = '\0';
	      optr++;
	    }
	  }
	  optr++;
	  d = atoi(tbuf);
	  /* min */
	  tbuf[0] = sbuf[optr];
	  optr++;
	  if (sbuf[optr] == ':') {
	    tbuf[1] = '\0';
	  } else {
	    tbuf[1] = sbuf[optr];
	    tbuf[2] = '\0';
	    optr++;
	  }
	  optr++;
	  m = atoi(tbuf);
	  /* sec */
	  for (i = 0; sbuf[optr] != '\0'; i++) {
	    tbuf[i] = sbuf[optr];
	    optr++;
	  }
	  tbuf[i] = '\0';
	  optr++;
	  s = (float)atof(tbuf);
	  if (d > 0) {
	    rad = (float)((d + m/60. + s/3600.) * PI / 180.);
	  } else {
	    rad = (float)((d - m/60. - s/3600.) * PI / 180.);
	  }
	  cobj->s_dec = rad;
	  break;
	case 5: /* mag */
	  cobj->s_mag = (float)atof(sbuf);
	  break;
	default:
	  break;
	}
	flg++;
	iptr++;
	optr = 0;
      } else {
	sbuf[optr] = buf[iptr];
	iptr++;
	optr++;
      }
    }

    cobj->o_type = FIXED;
    cobj->f_class = 'S';
    cobj->f_spect[0] = 'D'; /* dummy */
    cobj->f_spect[1] = '0'; /* dummy */
    cobj->f_size = 0.1;     /* dummy */


    if ((obj_line[entnum] = (char*)malloc(sizeof(char)*(iptr+2))) == NULL) {
      printf("out of memory. %d\n", entnum);
      return(-1);
    }
    strncpy(obj_line[entnum], buf, iptr-1);
    obj_line[entnum][iptr-1] = ',';
    obj_line[entnum][iptr] = '0';
    obj_line[entnum][iptr+1] = '\0';
    /*
      obj_line[entnum][iptr+2] = '1';
      obj_line[entnum][iptr+3] = '\0';
    */

    obj_edb[entnum] = cobj;
    entnum++;
  }

  return(entnum);
}



/* update AZ, EL */
int update_objazel(int num, GtkListStore *store) {
  int i;

  GtkTreeIter iter;
  Obj* cobj;


  for (i=0; i < num; i++) {
    cobj = obj_edb[i];

    gtk_list_store_append (store, &iter);
    gtk_list_store_set(store, &iter, 
		       OBJ_AZ,   cobj->s_az,
		       OBJ_DEC,  cobj->s_alt,
		       -1);
  }

  return(EXIT_SUCCESS);
}		       

/* update object list 
 * v0.5.8 2004.02.23 */
void update_object_list(void) {
  int i;

  /* free current data*/
  for (i = 0; i < mseemon.objnum; i++) {
    free(obj_edb[i]);
  }

  mseemon.objnum = read_edb(edbpath);

}

/* reset s_flags */
int reset_obj_flags(void) {
  int i;

  for (i = 0; i < mseemon.objnum; i++) {
    obj_edb[i]->o_flags = 0;
  }

  return(EXIT_SUCCESS);
}
