/*
 * 2002.11.08-
 * Time-stamp: <2004-02-23 14:41:57 dimm>
 */
#include <string.h>
#include <gtk/gtk.h>
#include "seemon2.h"

void dialog_destroy (GtkWidget *widget, gpointer data);
void click_cancel (GtkWidget *widget, gpointer data);
void click_ok (GtkWidget *widget, gpointer data);
void select_row_callback (GtkWidget *widget, gint row, gint col);

static int tmpsel = -1;

int init_obj_dialog(void) {
  int i;
  myTime mt;
  double d, tmjd, lst, t;
  float alt, mag;

  GtkWidget *dialog1;
  GtkWidget *button;
  GtkWidget *vbox1, *hbox1, *hbox2;
  GtkWidget *dialog_action_area;

  GtkWidget *list1;
  gchar *titles[4] = {"Name", "Alt", "H.A.", "Mag"};
  gchar *contents[4];
  gchar text[4][128];


  dialog1 = gtk_dialog_new();
  gtk_window_set_policy(GTK_WINDOW(dialog1), TRUE, TRUE, TRUE);

  gtk_object_set_data(GTK_OBJECT(dialog1), "obj_dialog", dialog1);
  gtk_window_set_title(GTK_WINDOW(dialog1), "Seemon2: select object");
  gtk_window_set_modal(GTK_WINDOW(dialog1), FALSE);

  gtk_signal_connect(GTK_OBJECT(dialog1), "destroy",
		     GTK_SIGNAL_FUNC(dialog_destroy), dialog1);

  /* pre-defined widgets */
  dialog_action_area = GTK_DIALOG(dialog1)->action_area;
  vbox1 = GTK_DIALOG(dialog1)->vbox;

  hbox2 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox2);
  gtk_box_pack_start(GTK_BOX(vbox1), hbox2, TRUE, TRUE, 5);

  list1 = gtk_clist_new_with_titles (4, titles);
  gtk_box_pack_start(GTK_BOX(hbox2), list1, TRUE, TRUE, 0);

  gtk_clist_set_selection_mode(GTK_CLIST(list1), GTK_SELECTION_SINGLE);
  // this does not work in gtk2?
  //  gtk_clist_set_policy (GTK_CLIST(list1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_clist_set_column_width (GTK_CLIST(list1), 0, 100);
  gtk_clist_set_column_width (GTK_CLIST(list1), 1, 30);
  gtk_clist_set_column_width (GTK_CLIST(list1), 2, 30);
  gtk_signal_connect (GTK_OBJECT(list1),  "select_row", GTK_SIGNAL_FUNC(select_row_callback), NULL);


  /* get current time */
  get_time(&mt);
  d = mt.day + mt.hour / 24. + mt.min / (24.*60.) + mt.sec / (24.*3600.);
  tmjd =  myCalcmjd(mt.year, mt.month, mt.day, mt.hour, mt.min);
  lst = get_lst(tmjd, msite.slon);

  #ifdef DEBUG
  printf("# current LST: %8.4f\n", lst);
  #endif

  for (i = 0; i < mseemon.objnum; i++) {
    get_azel (i, lst);

    /* 1st column : name */
    strcpy(text[0], obj_edb[i]->o_name);
    contents[0] = text[0];

    /* 2nd column : altitude */
    //    printf("%d %f\n", i, (obj_edb[i]->s_alt * 180 / PI));

    alt = (float)(obj_edb[i]->s_alt * 180 / PI);
    if (alt < 0) {
      text[1][0] = ' ';
      text[1][1] = '<';
      text[1][2] = '0';
      text[1][3] = ' ';
    } else {
      if (alt < 10) {
	text[1][0] = ' ';
	text[1][1] = 48 + (char)((int)alt % 10);
	text[1][2] = '.';
	text[1][3] = 48 + (char)((int)(alt * 10) % 10);
      } else {
	text[1][0] = 48 + (char)((int)alt / 10);
	text[1][1] = 48 + (char)((int)alt % 10);
	text[1][2] = '.';
	text[1][3] = 48 + (char)((int)(alt * 10) % 10);
      } 
    }
    text[1][4] = '\0';
    contents[1] = text[1];

    /* 3rd column : H.A. */
    t = (float)(obj_edb[i]->s_ha * 12. / PI);
    if (t < -12) t += 24.;
    if (t > 12) t -= 24.; /* added v0.5.8 2004.02.23 */
    if (t < 0) {
      t *= -1.;
      if (t > 10) {
	text[2][0] = '-';
	text[2][1] = 48 + (char)((int)t / 10);
	text[2][2] = 48 + (char)((int)t % 10);
	text[2][3] = '.';
	text[2][4] = 48 + (char)((int)(t * 10) % 10);
      } else {
	text[2][0] = ' ';
	text[2][1] = '-';
	text[2][2] = 48 + (char)((int)t % 10);
	text[2][3] = '.';
	text[2][4] = 48 + (char)((int)(t * 10) % 10);
      }
    } else {
      if (t < 10) {
	text[2][0] = ' ';
	text[2][1] = ' ';
	text[2][2] = 48 + (char)((int)t % 10);
	text[2][3] = '.';
	text[2][4] = 48 + (char)((int)(t * 10) % 10);
      } else {
	text[2][0] = ' ';
	text[2][1] = 48 + (char)((int)t / 10);
	text[2][2] = 48 + (char)((int)t % 10);
	text[2][3] = '.';
	text[2][4] = 48 + (char)((int)(t * 10) % 10);
      } 
    }
    text[2][5] = '\0';
    contents[2] = text[2];

    /* 4th column : mag */
    mag = obj_edb[i]->s_mag;

    if (mag >= 10.0) {
      text[3][0] = 48 + (char)(mag / 10.);
      text[3][1] = 48 + (char)((int)mag % 10);
      text[3][2] = '.';
      text[3][3] = 48 + (char)((int)(mag * 10) % 10);
    } else if (mag < 0) {
      text[3][0] = '-';
      text[3][1] = 48 + (char)((int)(-1 * mag) % 10);
      text[3][2] = '.';
      text[3][3] = 48 + (char)((int)(-1 * mag * 10) % 10);
    } else {
      text[3][0] = ' ';
      text[3][1] = 48 + (char)((int)mag % 10);
      text[3][2] = '.';
      text[3][3] = 48 + (char)((int)(mag * 10) % 10);
    }
    text[3][4] = '\0';
    contents[3] = text[3];

    gtk_clist_append(GTK_CLIST(list1), contents);
  }

  gtk_widget_show(list1);


  /* cancel button */
  button = gtk_button_new_with_label("Cancel");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX(dialog_action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(click_cancel), dialog1);

  /* select button */
  button = gtk_button_new_with_label("Select");
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX(dialog_action_area), button, TRUE, TRUE, 0);
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(click_ok), dialog1);
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button);

  gtk_widget_show(dialog1);
  gtk_grab_add(GTK_WIDGET(dialog1));

}


/******
 * callback functions for this dialog 
 ******/

/* close dialog */
void dialog_destroy (GtkWidget *widget, gpointer data)
{
  gtk_grab_remove(widget);
  //  gtk_widget_destroy(GTK_WIDGET(data));
  gtk_widget_destroy((GtkWidget*)data);
}

/* click cancel */
void click_cancel (GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(GTK_WIDGET(data));
}

/* click ok */
void click_ok (GtkWidget *widget, gpointer data)
{
  gchar text[128];
  gchar* content;

  if (tmpsel >= 0) {
    mseemon.selnum = tmpsel;
  }

  strcpy (text, obj_edb[mseemon.selnum]->o_name);
  content = text;
  gtk_entry_set_text (GTK_ENTRY (ent_selobj1), content);

  /* set target */
  set_target_by_object(); /* in telescope2.c */
  showTargetRA();
  showTargetDec();

  gtk_widget_destroy(GTK_WIDGET(data));
}

/* select a listed item */
void select_row_callback (GtkWidget *widget, gint row, gint col) {
  tmpsel = row;

  // check
  //  printf ("%d is selected.\n", tmpsel);
}
