#! /usr/bin/perl -w 
# 2003.05.01, 05.27
# 2003.10.04, 10.05
# 2004.02.20
# 2004.11.22
# 2004.12.06 modified path

use Math::Complex;
use PGPLOT;

$logpath = "/net/dimm/dimm1/log/";
$imgpath = "/net/dimm/dimm1/img/";

if ($ARGV[0] eq "") {
    printf "usage: pg_seemon2.pl <input> \n";
    exit;
}

$pi = atan(1)*4;

# observed wavelength (cm)
#$wv = 0.55e-4;

# FWHM threshold
$fth = 3.5;

# time threshold in sec.
$time_th = 60;
$time_th /= 3600.;

$infile  = $ARGV[0];
#$outfile = $ARGV[1];
$base = $infile;
$base =~ s/.log//;

open(IN, $logpath.$infile) || die "failed opening $infile\n";
#open(OUT, ">$outfile");

#printf OUT "#                 raw                 raw                 corrected\n";
#printf OUT "#time             ROX       R0Y       FHWMX     FWHMY     FHWMX     FWHMY\n";

#$prevtime = -1;
$n = 0;

while (<IN>) {
    if ($_ !~ /^\#/) {
	@fld = split;
	# get time
	$timestr = $fld[0];
	$th = substr($timestr, 11, 2);
	$tm = substr($timestr, 14, 2);
	$ts = substr($timestr, 17, 2);

	if ($th < 11) {
	    $th += 24;
	}
	$t[$n] = ($th*3600.+$tm*60.+$ts)/3600.;

	# r0
	$r0x[$n] = $fld[7];
	$r0y[$n] = $fld[8];
	# peak
	$peakx[$n] = $fld[18];
	$peaky[$n] = $fld[19];
	# altitude
	$alt[$n] = $fld[17];
	# shutter speed
	$sspeed[$n] = $fld[22]/100.;

	if ($r0x[$n] != 0 && $r0y[$n] != 0) {
	    # FWHM corrected
	    $fcx[$n] = $fld[11];
	    $fcy[$n] = $fld[12];
	    if ($fcx[$n] < $fth && 
		$fcy[$n] < $fth) {
		$n++;
	    }
	}
    }
}

close(IN);
#close(OUT);

if ($n == 0) {
    printf "no data points.\n";
    exit;
}

#$dev = "pgplot.png/png";

### plot: FWHM
$dev = "pgplot.eps/cps";

if (pgopen($dev) < 1) {
    printf "failed to open device file.\n";
    exit;
}

pgscf(1);       # Set label character font (1:normal,2:roman,3:italic,4:script)
pgslw(2);       # Set line width
pgsch(1.5);     # Set label character height
pgsci(1);

# display range
$x0 = 17.0;
$x1 = 29.5;
$y0 =  0.0;
$y1 =  2.1;

pgenv($x0, $x1, $y0, $y1, 0, 0);

$str = "input:".$infile;
pglabel("Time (JST)", "corrected FWHM", $str);

pgsch(0.5);
pgsci(4);
pgpt($n, *t, *fcx, 17);
pgsci(2);
pgpt($n, *t, *fcy, 17);


pgsci(1);
pgclos();

$eps = "pgplot.eps";
$png = $imgpath.$base."_fwhm.png";
system("/usr/bin/convert -rotate 90 $eps $png");


### plot: Peak
$dev = "pgplot.eps/cps";

if (pgopen($dev) < 1) {
    printf "failed to open device file.\n";
    exit;
}

pgscf(1);       # Set label character font (1:normal,2:roman,3:italic,4:script)
pgslw(2);       # Set line width
pgsch(1.5);     # Set label character height
pgsci(1);

# display range
$x0 = 17.5;
$x1 = 29.5;
$y0 =  0.0;
$y1 = 150.;

pgenv($x0, $x1, $y0, $y1, 0, 0);

$str = "input:".$infile;
pglabel("Time (JST)", "Peak / Altitude / Shutter Speed", $str);

pgsch(0.5);
pgsci(4);
pgpt($n, *t, *peakx, 17);
pgsci(2);
pgpt($n, *t, *peaky, 17);
pgsci(3);
pgpt($n, *t, *alt, 17);
pgsci(8);
pgpt($n, *t, *sspeed, 17);

pgsci(1);
pgclos();

$eps = "pgplot.eps";
$png = $imgpath.$base."_peak.png";
system("/usr/bin/convert -rotate 90 $eps $png");

