#!/usr/bin/perl -w
# 2004.12.07

$thisfile = "scr.dimmstat01.pl";
$loglist  = "loglist.txt";

# FWHM threshold
$th = 10;

# clipping sigma
$limit = 3;

if ($ARGV[5] eq "") {
    printf "usage: $thisfile <year1> <month1> <date1> <year2> <month2> <date2>\n";
    exit;
}

$year1  = $ARGV[0];
$month1 = $ARGV[1];
$date1  = $ARGV[2];
$year2  = $ARGV[3];
$month2 = $ARGV[4];
$date2  = $ARGV[5];

$t1 = $year1.$month1.$date1;
$t2 = $year2.$month2.$date2;

open(IN, $loglist) || die "failed in opening $loglist";
$n = 0;
while (<IN>) {
    if ($_ !~ /^\#/) {
	@fld = split;
	chomp();
	$t = substr($_, 6, 8);

	if ($t >= $t1 && $t <= $t2) {
	    $infile[$n] = $_;
	    $n++;
	}
    }
}
close(IN);

$m = 0;
for ($i = 0; $i < $n; $i++) {
    open(IN, $infile[$i]) || die "failed in opening $infile[$i]";
    while (<IN>) {
	if ($_ !~ /^\#/) {
	    @fld = split;
	    $tx = $fld[11];
	    $ty = $fld[12];
	    if ($tx > 0 && $ty > 0 && $tx < $th && $ty < $th) {
		$f1[$m] = $tx;
		$f2[$m] = $ty;
		$m++;
	    }
	}
    }
}

printf "#number of data points: %d.\n", $m;

@fs1 = sort{$a <=> $b} @f1;
@fs2 = sort{$a <=> $b} @f2;

printf "# finished sorting.\n";

for ($i = 0; $i < $m; $i++) {
    $fc[$i][0] = $fs1[$i];
    $fc[$i][1] = $fs2[$i];
    $flg[$i][0] = 1;
    $flg[$i][1] = 1;
}

for ($k = 0; $k < 2; $k++) {
    $prevnum = -1;
    $loopflg = 1;
    while ($loopflg) {
	# copy data
	$pt = 0;
	for ($i = 0; $i < $m; $i++) {
	    if ($flg[$i][$k]) {
		$x[$pt] = $fc[$i][$k];
		$pt++;
	    }
	}
	$num = $pt;

	# mean
	$tot = 0.;
	for ($i = 0; $i < $num; $i++) {
	    $tot += $x[$i];
	}
	$mean = ($tot * 1.0) / $num;

	# stddev
	$tot = 0.;
	for ($i = 0; $i < $num; $i++) {
	    $y = $x[$i] - $mean;
	    $tot += $y**2;
	}
	$stddev = sqrt($tot/$num);

	# median
	$y = $num % 2;
	if ($y == 0) {
	    # num is even
	    $y = $num / 2;
	    $median = ($x[$y] + $x[$y+1]) / 2.;
	} else {
	    # num is odd
	    $y = $num /2 + 1;
	    $median = $x[$y];
	}

	if ($num == $prevnum) {
	    $loopflg = 0;
	} else {
	    $prevnum = $num;
	    $low = $median - $limit * $stddev;
	    $hgh = $median + $limit * $stddev;

	    for ($i = 0; $i < $m; $i++) {
		if ($fc[$i][$k] < $low || $fc[$i][$k] > $hgh) {
		    $flg[$i][$k] = 0;
		}
	    }
	}
    }
    $fmean[$k] = $mean;
    $fsig[$k] = $stddev;
    $fmed[$k] = $median;
    $fnum[$k] = $num;

    # histgram
    $h0 = 0.;
    $h1 = 4.;
    $step = 0.1;

    $x1 = $h0;
    $x2 = $x1 + $step;
    $loopflg = 1;
    $n = 0;
    $i = 0;
    while ($loopflg) {
	$hnum[$n][$k] = 0;
	while ($x[$i] <= $x2) {
	    $hnum[$n][$k]++;
	    $i++;
	}
	$x1 += $step;
	$x2 = $x1 + $step;
	$n++;
	if ($x2 > $h1) {
	    $loopflg = 0;
	}
    }

}

printf "# num mean median sigma \n";
for ($k = 0; $k < 2; $k++) {
    printf "%7d %6.3f %6.3f %6.3f \n", $fnum[$k], $fmean[$k], $fmed[$k], $fsig[$k];
}

$h0 = 0.;
$h1 = 4.;
$step = 0.1;

$x1 = $h0;
$x2 = $x1 + $step;
$loopflg = 1;
$n = 0;
while ($loopflg) {
    printf "%4.1f %4.1f %5d %5.3f %5d %5.3f \n", 
    $x1, $x2, $hnum[$n][0], $hnum[$n][0]/$num, $hnum[$n][1], $hnum[$n][1]/$num;
    $n++;
    $x1 += $step;
    $x2 = $x1 + $step;
    if ($x2 > $h1) {
	$loopflg = 0;
    }
}

