/* 
 * telescope2.c
 * 
 * for meade lx200 control
 * based on codes of xmtel
 *
 * 2003.02.03 -
 * Time-stamp: <2005-11-17 19:16:03 dimm>
*/
#include <stdlib.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "seemon2.h"
#include "protocol.h"

/*void showTelRA();          /* display latest telescope RA  */
/*void showTelDec();         /* display latest telescope Dec */

void showTargetRA();       /* display latest target RA  */
void showTargetDec();      /* display latest target Dec */

void fetchTelRA();         /* import current telescope RA */
void fetchTelDec();        /* import current telescope Dec */

int  linkTel();            /* startup telescope link routine */
void unlinkTel();          /* shutdown telescope link routine */

void slewTel();            /* slew telescope to target RA and Dec */
void resetTel();           /* reset telescope coordinates to target coordinates */

int dmstod();              /* convert string deg:min:sec to double */
void dtodms();             /* convert dms double to string deg:min:sec */


/* The following are provided by protocol.c */
extern void ConnectTel(char *port);
extern void DisconnectTel(void);
extern int  CheckConnectTel(void);

extern void SetRate(int newRate);
extern void SetLimits(double limitLower, double limitHigher);
extern void StartSlew(int direction);
extern void StopSlew(int direction);
extern double GetRA(void);
extern double GetDec(void);
extern int  SlewToCoords(double newRA, double newDec);
extern int  SyncToCoords(double newRA, double newDec);
extern int  CheckCoords(double desRA, double desDec);

/*double telra, teldec;                 /* telescope ra and dec */
/*double  targetra, targetdec;          /* target ra and dec */
double limitlower, limithigher;       /* slew limits */
/*int telspd;                           /* drive speed */
int teldir;                           /* drive direction */                        
int target;                           /* flag for target input */

int fd_fifo_in, fd_fifo_out;          /* fifo file descriptors */
FILE *fp_history;                     /* history file pointer */
char telhistory[20]="telhistory.log";     /* history file name    */


char buf[256];                        /* general purpose character buffer */

/* Start telescope communications */
int linkTel()
{
  int okflag = 0;
  
  // change port according to DIMM ID
  /* v0.7.7 dimm_id swapped */
  if (mseemon.dimm_id == 1) {
    ConnectTel("/dev/ttyS0");
  } else {
    ConnectTel("/dev/ttyUSB0");
  }
  fflush(stdout);
  okflag=CheckConnectTel();

  /* Connection diagnostics  ... */
  if (okflag == 1) {
    mseemon.tel_stat = 1;
    gtk_button_set_label(GTK_BUTTON(but_tel_conn), "Disconnect Telescope");
  } else {
    printf("Failed connecting to the telescope.\n");
    mseemon.tel_stat = 0;
    return(EXIT_FAILURE);
  }

  limitlower=LOWER;
  limithigher=HIGHER;
  SetLimits(limitlower,limithigher);
  /*  telspd=SLEW;           2003.02.07 */
  /*  SetRate(telspd);       2003.02.07 */
  SetRate(mseemon.tel_step);       
  fetchTelRA();
  showTelRA();
  fetchTelDec();
  showTelDec();
  mseemon.trg_ra=mseemon.tel_ra;
  showTargetRA();
  mseemon.trg_dc=mseemon.tel_dc;
  showTargetDec();

  return(EXIT_SUCCESS);
}

/* Stop telescope communications */
void unlinkTel()          
{
  fflush(stdout);
  DisconnectTel();

  mseemon.tel_stat = 0;
  gtk_button_set_label(GTK_BUTTON(but_tel_conn), "Connect to Telescope");

}

/* Import current telescope Right Ascension */
void fetchTelRA() 
{
  /* hours --> radian. corrected 2003.03.11 */
   mseemon.tel_ra=GetRA() * PI/12.;
}
 
/* Import current telescope Declination */
void fetchTelDec() 
{
   mseemon.tel_dc=GetDec() * PI/180.;
}

/* Show telescope RA coordinates in the window */
void showTelRA()
{
  double ra_h;
  gchar text[10];

  ra_h = mseemon.tel_ra * 15. / PI;
  dtodms(buf, &ra_h);
  strcpy(text, buf);

  gtk_entry_set_text(GTK_ENTRY(ent_tel_ra), text);
  
  /*
    XmString xmstr;
    Arg al[10];
    int ac;

    dtodms(buf, &telra);
    xmstr = XmStringCreateLtoR(buf,char_set);
    ac=0;
    XtSetArg(al[ac],XmNlabelString,xmstr); ac++;
    XtSetValues(label_telra,al,ac);   
    XmStringFree(xmstr);
  */
}    

/* Show telescope Dec coordinates in the window */
void showTelDec()
{
  double dc_d;
  gchar text[10];

  dc_d = mseemon.tel_dc * 180. / PI;
  dtodms(buf, &dc_d);
  strcpy(text, buf);

  gtk_entry_set_text(GTK_ENTRY(ent_tel_dc), text);
  
  /*
    XmString xmstr;
    Arg al[10];
    int ac;

    dtodms(buf, &teldec);
    xmstr = XmStringCreateLtoR(buf,char_set);
    ac=0;
    XtSetArg(al[ac],XmNlabelString,xmstr); ac++;
    XtSetValues(label_teldec,al,ac);   
    XmStringFree(xmstr);
  */
}       
      
/* Show target RA coordinates in the window */

void showTargetRA()
{
  double ra_h;
  gchar text[10];

  ra_h = mseemon.trg_ra * 12. / PI;
  dtodms(buf, &ra_h);
  strcpy(text, buf);

  gtk_entry_set_text(GTK_ENTRY(ent_trg_ra), text);
  
  /*
    XmString xmstr;
    Arg al[10];
    int ac;

    dtodms(buf, &targetra);
    xmstr = XmStringCreateLtoR(buf,char_set);
    ac=0;
    XtSetArg(al[ac],XmNlabelString,xmstr); ac++;
    XtSetValues(label_targetra,al,ac);   
    XmStringFree(xmstr);
  */
}    
  

/* Show target Dec coordinates in the window */
void showTargetDec()
{
  double dc_d;
  gchar text[10];

  dc_d = mseemon.trg_dc * 180. / PI;
  dtodms(buf, &dc_d);
  strcpy(text, buf);

  gtk_entry_set_text(GTK_ENTRY(ent_trg_dc), text);
  
  /*
    XmString xmstr;
    Arg al[10];
    int ac;

    dtodms(buf, &targetdec);
    xmstr = XmStringCreateLtoR(buf,char_set);
    ac=0;
    XtSetArg(al[ac],XmNlabelString,xmstr); ac++; 
    XtSetValues(label_targetdec,al,ac);   
    XmStringFree(xmstr);
  */
}       


/* Convert string deg:min:sec or hr:min:sec to a double */
int dmstod (instr, datap)
char *instr;
double *datap;
{
  double h=0., m=0., s=0.;
  int negflag;
  int convertflag;
  
  while (isspace(*instr))
    instr++;
  if (*instr == '-') {
    negflag = 1;
    instr++;
  } else
    negflag = 0;
    
  convertflag = sscanf (instr,  "%lf%*[:]%lf%*[:]%lf", &h, &m, &s);
  if (convertflag < 1)
    return (-1);
  *datap = h + m/60. + s/3600.;
  if (negflag)
    *datap = - *datap;
  return (0);
}

/* Convert double to string deg:min:sec or hr:min:sec */
void dtodms (outstr, dmsp)
char *outstr;
double *dmsp;
{

  int d=0, m=0, s=0;
  int negflag;
  int convertflag;
  double dms, ms;
   
  dms = *dmsp;

  /* If negative, treat as positive and set sign in string */
  if (dms < 0 ) {
    negflag = 1;
    dms = - dms;
  }
  else
    negflag = 0;


  /* Allow for truncation if input is in seconds of arc */
  dms = dms + 0.5/3600.;

  /* Get the whole part in degrees or hours, as needed */  
  d = (int) dms;
  
  /* Could test here for overflow at 24h or 360d          */
  /*   but we'll assume that's been done before the call. */
  ms = dms - (double) d;
  ms = ms*60.;
  m = (int) ms;
  s = (int) 60.*(ms - (double) m);
  if (negflag)
  {
    sprintf(outstr,"-%02d:%02d:%02d",d,m,s);
  }
  else
  {
    sprintf(outstr,"%02d:%02d:%02d",d,m,s);      
  }
}

/* Slew telescope to target coordinates */
/* added timeout v0.4.8 20030909 iwata */
void slewTel()
{
  double targetra, targetdec;
  int slewflag=0;
  int cnt = 0;

  /* in hms and dms 
   * iwata 20030204 */
  targetra  = mseemon.trg_ra * 12./PI;
  targetdec = mseemon.trg_dc * 180./PI;

  #ifdef DEBUG
  printf("target coords: %8.4f %8.4f\n", targetra, targetdec);
  #endif

  slewflag=SlewToCoords(targetra,targetdec);
  #ifdef DEBUG
  printf("slewflag=%d\n", slewflag);
  #endif

  //  sleep(10);
  usleep(10000000);

  while( !CheckCoords(targetra,targetdec) )
    {
      #ifdef DEBUG
      printf("checking coords...\n");
      #endif
      usleep(5000000);

      /* update coords */
      fetchTelRA();
      fetchTelDec();
      showTelRA();
      showTelDec();

      cnt++;
      if (cnt > 10) {
	printf("#pointing timeout.\n");
	break;
      }

      /* export to XEphem disabled 
      if (fd_fifo_out>=0) 
        markXEphemTel();
      */
    }

  /* wait 10 sec as safety buffer */
  printf("# slewTel: waiting 10 sec.\n");
  usleep(10000000);

  #ifdef DEBUG
  printf("pointing finished\n");
  #endif


}           


/* Reset telescope coordinates to target coordinates */
void resetTel()          
{    
  double targetra, targetdec;
  int syncflag=0;

  /* in hms and dms 
   * iwata 20030204 */
  targetra  = mseemon.trg_ra * PI/15.;
  targetdec = mseemon.trg_dc * PI/180.;

  syncflag=SyncToCoords(targetra,targetdec);
  usleep(500000.);
}


/* set target ra and dec
 * according to a selected object 
 */
void set_target_by_object(void) {
  Obj *op;

  if (mseemon.selnum < 0) {
    printf("no object is selected.\n");
    return;
  }

  op = obj_edb[mseemon.selnum];

  mseemon.trg_ra = op->s_ra;
  mseemon.trg_dc = op->s_dec;

  return;
}

